"""
Pyrex/C extension supporting `bx.misc.seekbzip2` (wrapping the low level
functions in `micro-bunzip.c`).
"""

cdef extern from "Python.h":
    char * PyBytes_AsString( object )
    object PyBytes_FromStringAndSize( char *, Py_ssize_t )

cdef extern from "micro-bunzip.h":
    ctypedef struct bunzip_data:
        int in_fd
        int inbufBitCount
        int inbufPos
        int inbufCount
        int writeCount
        unsigned int writeCRC
        int writeCurrent
        int writeCopies
        unsigned int * dbuf
    unsigned int get_bits(bunzip_data *bd, char bits_wanted)
    int get_next_block( bunzip_data *bd )
    int read_bunzip(bunzip_data *bd, char *outbuf, int len)
    int start_bunzip(bunzip_data **bdp, int in_fd, char *inbuf, int len)
    int read_bunzip_to_char(bunzip_data *bd, char *outbuf, int len, int* gotcount_out, char stopchar )
    
cdef extern from "unistd.h":
    # Not really
    ctypedef unsigned long long off_t
    off_t lseek( int fildes, off_t offset, int whence )
    
cdef extern from "stdlib.h":
    void free( void *ptr )
 
import sys
import os
    
cdef class SeekBzip2:

    cdef bunzip_data * bd
    cdef int file_fd
    cdef int at_eof

    def __init__( self, filename ):
        self.at_eof = 0
        self.file_fd = os.open( filename, os.O_RDONLY )
        # Initialize bunzip_data from the file
        start_bunzip( &( self.bd ), self.file_fd, NULL, 0 )
        
    def close( self ):
        free( self.bd.dbuf )
        free( self.bd )
        os.close( self.file_fd )

    def seek( self, unsigned long long position ):
        """
        Seek the bunzip_data to a specific chunk (position must correspond to
        that start of a compressed data block).
        """
        cdef off_t n_byte
        cdef int n_bit
        # Break position into bit and byte offsets
        ## sys.stderr.write( "arg pos: %d\n" % position )
        n_byte = position / 8;
        n_bit = position % 8;
        ## sys.stderr.write( "byte pos: %d\n" % n_byte )
        ## sys.stderr.write( "bit pos: %d\n" % n_bit )
        ## sys.stderr.flush()
        # Seek the underlying file descriptor
        if ( lseek( self.file_fd, n_byte, 0 ) != n_byte ):
            raise Exception( "lseek of underlying file failed" )
        # Init the buffer at the right bit position
        self.bd.inbufBitCount = self.bd.inbufPos = self.bd.inbufCount = 0
        get_bits( self.bd, n_bit )
        # This ensures that the next read call will return 0, causing the
        # buffer to be re-initialized
        self.bd.writeCount = -1
        # Reset EOF tracking
        self.at_eof = 0
        
    def readline( self, int amount ):
        cdef object rval
        cdef char * p_rval
        cdef int gotcount
        cdef int totalcount
        cdef int status
        cdef int spaceleft
        cdef int desired
        gotcount = 0
        totalcount = 0
        # If already at EOF return None
        if self.at_eof:
            return None
        chunks = []
        # We have great difficulty resizing buffers, so we'll just create
        # one 8k string at a time
        rval = PyBytes_FromStringAndSize( NULL, 8192 )
        p_rval = PyBytes_AsString( rval )
        spaceleft = 8192
        while amount != 0:
            if amount > 0 and amount < spaceleft:
                desired = amount
            else:
                desired = spaceleft
            ## sys.stderr.write( "readline, amount: %d\n" % amount )
            ## sys.stderr.write( "buffer: %r" % rval[:100] )
            ## sys.stderr.write( "\n" )
            ## sys.stderr.flush()
            # ord( "\n" ) = 10
            status = read_bunzip_to_char( self.bd, p_rval, desired, &gotcount, 10 );
            ## sys.stderr.write( "readline, desired: %d, gotcount: %d\n" % ( desired, gotcount ) ); 
            ## sys.stderr.write( "buffer: %r" % rval[:100] )
            ## sys.stderr.write( "\n" )
            ## sys.stderr.flush()
            if status == -9: 
                ## sys.stderr.write( "readline, STOP_CHAR\n" ); sys.stderr.flush()
                # Reached the stop character (RETVAL_STOPCHAR == -9), so 
                # we can stop
                chunks.append( rval[:8192-spaceleft+gotcount] )
                break
            elif status == -10:
                ## sys.stderr.write( "readline, BUFFER_FULL\n" ); sys.stderr.flush()
                # Filled the buffer (RETVAL_BUFFER_FULL == -10), so create
                # new buffer and keep going
                chunks.append( rval )
                amount = amount - gotcount
                if amount == 0:
                    # Got the desired amount
                    break
                rval = PyBytes_FromStringAndSize( NULL, 8192 )
                p_rval = PyBytes_AsString( rval )
                spaceleft = 8192
            elif status == -8:
                ## sys.stderr.write( "readline, END_OF_BLOCK\n" ); sys.stderr.flush()
                # No more data in the decomp buffer (RETVAL_END_OF_BLOCK == -10)
                if gotcount and p_rval[ gotcount - 1 ] == 10:
                    chunks.append( rval[:8192-spaceleft+gotcount] )
                    break
                # Update buffer info
                p_rval = p_rval + gotcount
                spaceleft = spaceleft - gotcount
                amount = amount - gotcount
                # Get the next block
                status = get_next_block( self.bd )
                if status == -1:
                    # Block is end of stream block (RETVAL_LAST_BLOCK == -1)
                    self.at_eof = 1
                    chunks.append( rval[:gotcount] )
                    break
                self.bd.writeCRC = 0xffffffff
                self.bd.writeCopies = 0
            else:
                # Some other status
                raise Exception( "read_bunzip error %d" % status )
        # Return whatever we read
        return "".join( chunks )
        
    def read( self, int amount ):
        cdef object rval
        cdef char * p_rval
        cdef int gotcount
        cdef int totalcount
        cdef int status
        totalcount = 0
        # If already at EOF return None
        if self.at_eof:
            return None
        # Create a new python bytes string large enough to hold the result
        rval = PyBytes_FromStringAndSize( NULL, amount )
        p_rval = PyBytes_AsString( rval )
        # Read into it
        ## sys.stderr.write( "read called, bd.current: %x\n" % self.bd.writeCurrent ); sys.stderr.flush()
        while amount > 0:
            gotcount = read_bunzip( self.bd, p_rval, amount );
            if gotcount < 0:
                raise Exception( "read_bunzip error %d" % gotcount )
            elif gotcount == 0:
                status = get_next_block( self.bd )
                if status == -1:
                    self.at_eof = 1
                    break
                self.bd.writeCRC = 0xffffffff
                self.bd.writeCopies = 0
            else:
                totalcount = totalcount + gotcount
                amount = amount - gotcount
                p_rval = p_rval + gotcount
        # Return whatever we read
        return rval[:totalcount]



#!/usr/bin/env python

"""
For each block in a maf file (read from stdin) write a sequence of ints
corresponding to the columns of the block after applying the provided sequence
mapping.

The 'correct' number of species is determined by the mapping file, blocks not having
this number of species will be ignored.

usage: %prog mapping_file
"""
from __future__ import division, print_function

import sys

import bx.align.maf
from bx import seqmapping


def main():

    if len(sys.argv) > 1:
        _, alpha_map = seqmapping.alignment_mapping_from_file(open(sys.argv[1]))
    else:
        alpha_map = None

    for maf in bx.align.maf.Reader(sys.stdin):
        # Translate alignment to ints
        int_seq = seqmapping.DNA.translate_list([c.text for c in maf.components])
        # Apply mapping
        if alpha_map:
            int_seq = alpha_map.translate(int_seq)
        # Write ints separated by spaces
        for i in int_seq:
            print(i, end=' ')
        print()


if __name__ == "__main__":
    main()



#!/usr/bin/python2.4
from __future__ import print_function

import sys

import bx.pwm.position_weight_matrix as pwmx
from bx.align import maf as align_maf


def isnan(x):
    # return ieeespecial.isnan(x)
    if x == x:
        return False
    return True


NaN = float('nan')


def main():

    pwm_file = sys.argv[1]
    splist = sys.argv[2]
    if len(sys.argv) == 4:
        inmaf = open(sys.argv[3])
    else:
        inmaf = sys.stdin

    # read alignment species
    species = []
    for sp in splist.split(','):
        species.append(sp)

    # read weight matrices
    pwm = {}
    for wm in pwmx.Reader(open(pwm_file), format='basic'):
        pwm[wm.id] = wm

    fbunch = {}
    for scoremax, index, headers in MafScorer(pwm, species, inmaf):
        for k, matrix in scoremax.items():
            fname = k + '.mx'
            if fname not in fbunch:
                fbunch[fname] = open(fname, 'w')
                print("Writing", fname, file=sys.stderr)

            for i in range(len(matrix)):
                for j in range(len(matrix[i])):
                    print("%.2f" % matrix[i][j], end=' ', file=fbunch[fname])
                print(file=fbunch[fname])

    for file in fbunch.values():
        file.close()


def MafScorer(pwm, species, inmaf):

    index = 0
    scoremax, width = None, None
    for maf in align_maf.Reader(inmaf):
        # try:
        if True:
            val = MafBlockScorer(pwm, species, maf)
            for scoremax, width, headers in val:
                yield scoremax, index, headers
        try:
            pass
        except Exception:
            print("Failed on:", file=sys.stderr)
            syserr = align_maf.Writer(sys.stderr)
            syserr.write(maf)
            if width:
                print(width, file=sys.stderr)
            if scoremax:
                print(len(scoremax), file=sys.stderr)
            syserr.close()
            sys.exit(1)
        index += width
        yield scoremax, index, headers


def MafMotifSelect(mafblock, pwm, motif=None, threshold=0):

    if motif is not None and len(motif) != len(pwm):
        raise Exception("pwm and motif must be the same length")
    # generic alignment
    alignlist = [c.text for c in mafblock.components]
    align = pwmx.Align(alignlist)
    nrows, ncols = align.dims
    # required sequence length
    minSeqLen = len(motif)
    # record the text sizes from the alignment rows

    for start in range(ncols - minSeqLen):
        if align.rows[0][start] == '-':
            continue
        subseq = ""
        pwm_score_vec = []
        motif_score_vec = []
        max_cols = 0
        for ir in range(nrows):
            expanded = align.rows[ir].count('-', start, minSeqLen)
            subtext = align.rows[ir][start: minSeqLen+expanded]
            max_cols = max(len(subtext), max_cols)
            subseq = subtext.replace('-', '')
            revseq = pwmx.reverse_complement(subseq)
            # pwm score
            nill, f_score = pwm.score_seq(subseq)[0]
            r_score, nill = pwm.score_seq(revseq)[0]
            pwm_score_vec.append(max(f_score, r_score))
            # consensus score
            if motif is not None:
                for_score = int(pwmx.match_consensus(subseq, motif))
                rev_score = int(pwmx.match_consensus(revseq, motif))
                motif_score_vec.append(max(for_score, rev_score))
        # check threshold
        try:
            assert not isnan(max(pwm_score_vec))
            assert not isnan(max(motif_score_vec))
        except AssertionError:
            print(pwm_score_vec, motif_score_vec, file=sys.stderr)
            print(len(subseq), len(pwm), file=sys.stderr)
        if max(pwm_score_vec) < threshold:
            continue
        if max(motif_score_vec) < threshold:
            continue
        # chop block
        col_start = start
        col_end = max_cols + 1
        motifmaf = mafblock.slice(col_start, col_end)
        yield motifmaf, pwm_score_vec, motif_score_vec

    """
    for ir in range(nrows):
        # scan alignment row for motif subsequences
        for start in range(ncols):
            if align.rows[ir][start] == '-': continue
            elif align.rows[ir][start] == 'n': continue
            elif align.rows[ir][start] == 'N': continue
            # gather enough subseq for motif
            for ic in range(start,ncols):
                char = align.rows[ir][ic].upper()
                if char == '-' or char == 'N': continue
                else: subseq += char
                if len(subseq) == minSeqLen:
                    revseq = pwmx.reverse_complement( subseq )
                    align_match_lens.append( ic )
                    # pwm score
                    nill,f_score = pwm.score_seq( subseq )[0]
                    r_score, nill = pwm.score_seq( revseq )[0]
                    pwm_score_vec.append( max(f_score, r_score) )
                    # consensus score
                    if motif is not None:
                        for_score = int( pwmx.match_consensus(subseq,motif) )
                        rev_score = int( pwmx.match_consensus(revseq,motif) )
                        motif_score_vec.append( max(for_score, rev_score) )
                    #check threshold
                    try:
                        assert not isnan(max(pwm_score_vec) )
                        assert not isnan(max(motif_score_vec) )
                    except:
                        print >>sys.stderr, pwm_score_vec, motif_score_vec
                        print >>sys.stderr, len(subseq), len(pwm)
                    if max(pwm_score_vec) < threshold: continue
                    if max(motif_score_vec) < threshold: continue
                    # chop block
                    col_start = start
                    col_end = max( align_match_lens ) + 1
                    motifmaf = mafblock.slice( col_start, col_end )

                    print subseq,revseq,ic
                    print align_match_lens
                    yield motifmaf, pwm_score_vec, motif_score_vec
        """


def MafBlockScorer(pwm, species, maf):
    width = len(maf.components[0].text)
    headers = [(c.src, c.start, c.end) for c in maf.components]

    # expand block rows to full
    mafBlockSpecies = [specName.src.split('.')[0] for specName in maf.components]
    alignlist = []
    for sp in species:
        try:
            i = mafBlockSpecies.index(sp)
            alignlist.append(maf.components[i].text)
        except ValueError:
            alignlist.append([NaN for n in range(width)])
    alignrows = pwmx.Align(alignlist)
    scoremax = {}
    # record gap positions
    filter = pwmx.score_align_gaps(alignrows)
    # score pwm models
    for model in pwm.keys():
        scoremax[model] = pwm[model].score_align(alignrows, filter)
    yield scoremax, width, headers


def MafMotifScorer(species, maf, motifs):
    width = len(maf.components[0].text)
    headers = [(c.src, c.start, c.end) for c in maf.components]

    # expand block rows to full
    mafBlockSpecies = [specName.src.split('.')[0] for specName in maf.components]
    alignlist = []
    for sp in species:
        try:
            i = mafBlockSpecies.index(sp)
            alignlist.append(maf.components[i].text)
        except ValueError:
            alignlist.append([NaN for n in range(width)])

    alignrows = pwmx.Align(alignlist, headers)
    # record gap positions
    filter = pwmx.score_align_gaps(alignrows)
    # score motif
    if isinstance(motifs, list):
        scoremax = {}
        for string in motifs:
            scoremax[string] = pwmx.score_align_motif(alignrows, string, filter)
    else:
        scoremax = pwmx.score_align_motif(alignrows, motifs, filter)
    yield scoremax, width, headers


if __name__ == '__main__':
    main()



"""
Classes to support FASTA files.

:Author: Bob Harris (rsharris@bx.psu.edu)

A FASTA file contains multiple sequences.  Each sequence is usually DNA.

A typical FASTA file::

   >mule
   TAATACCCCGGATATATGTCCTCACATAGTTCGAGGTCGAGAAAAATGAC
   TTCCCACCAAGTGGACTCAGCTCGAGTAAACGCCAACGATACGTCCATTA
   GGTGTGTGCCgcaactagtcggacccgttgtgacggaaacaggtccccgc
   caagtcacacgggcatgtcatggacTCTCGATCGTTCATCGCCTTCTTGG
   GTACCGCAGCCGCAATTAAGCCGTGTCTTCTTCCCCCTTCAAACGGGAAT
   CGTGTCGACTTCTTAGGAGCAGNNNNNNNNNNCTAACTCCAGAG
   >donkey
   TAATACCCCGGATATATGTCTTAACATAGTTCCAGGTCGAGAAGAATGAC
   TTGCCACCAAGTGGACTCAGATTCAGTCAACGCGAACGATAAGTCCATTA
   GGTGTGTACCgcaactagtgggacccgttgtgacggaaacaggtcaccgc
   caagtcacacgtgcatgtcatgtacTCTCGATCGTTTATCGCCTTCTTGG
   GTACCGCAGCCGAAATTAAGCCGTGTCTTCTTCCCACTTCAAACGGGAAT
   CGTGTCGACTTTACAGGAACAGNNNNNNNNNNATAACGCCAGAG
    ... more sequences

Typical use:

    for seq in bx.seq.fasta.FastaReader(sys.stdin):
        print seq.name
        print seq.get(0,seq.length)

"""
from __future__ import print_function


from bx.seq.seq import SeqFile, SeqReader


class FastaFile(SeqFile):

    def __init__(self, file, revcomp=False, name="", gap=None, lookahead=None, contig=None):
        SeqFile.__init__(self, file, revcomp, name, gap)
        self.lookahead = lookahead
        if (contig is None):
            contig = 1
        assert (contig >= 1), "contig %d is not legal" % contig

        # nota bene: certainly not the most efficient or elegant implementation

        currContig = 1
        while (True):
            if (self.lookahead is not None):
                (line, self.lookahead) = (self.lookahead, None)
            else:
                line = self.file.readline()
                if not isinstance(line, str):
                    line = line.decode()
            if (line == ""):
                break
            if not line:
                break
            if (line.startswith(">")):
                if (self.text is not None):
                    if (currContig == contig):
                        self.lookahead = line  # (next sequence header)
                        break
                    currContig += 1
                self.name = self.extract_name(line[1:])
                self.text = []
                continue
            line = line.split()  # (remove whitespace)
            if (self.text is None):
                self.text = line  # (allows headerless fasta)
            else:
                self.text.extend(line)
        assert (currContig == contig), \
            "contig %d is not legal (file contains only %d)" % (contig, currContig)
        if (self.text is not None):
            self.text = "".join(self.text)
            self.length = len(self.text)


class FastaReader(SeqReader):

    def __init__(self, file, revcomp=False, name="", gap=None):
        SeqReader.__init__(self, file, revcomp, name, gap)
        self.lookahead = None

    def __next__(self):
        seq = FastaFile(self.file, self.revcomp, self.name, self.gap, self.lookahead)
        if (seq.text is None):
            return
        self.lookahead = seq.lookahead
        self.seqs_read += 1
        return seq


class FastaWriter(object):

    def __init__(self, file, columns=50):
        self.file = file
        self.columns = columns

    def write(self, seq):
        print(">%s" % seq.name, file=self.file)
        text = seq.text
        if (self.columns is not None) and (self.columns > 0):
            text = "\n".join([text[ix:ix+self.columns]
                              for ix in range(0, len(text), self.columns)])
        print(text, file=self.file)

    def close(self):
        assert (self.file is not None)
        self.file.close()
        self.file = None



"""
Tests for `bx.align.lav`.
"""

import unittest

import bx.align.lav as lav

test_lav = "test_data/lav_tests/apple_orange.lav"


class lavTestCase(unittest.TestCase):

    def testReader(self):

        reader = lav.Reader(open(test_lav))

        a = next(reader)
        assert a.score == 10286, "a.score is wrong: %s" % a.score
        assert len(a.components) == 2
        check_component(a.components[0], "apple", 106, 252, "+", 411, "GTCCGGCCGGCTGAGAGCTACAATACACATGCACGCAGTTTGGCCACTCACATTAAGTATATGAGGAAGGGTTAGCATGAGTTGTACTATAAGGCAGCGGATAGCAGGTTGTGGAAAAATATCCTCCCGATTCAAATCCCCAGGTGCCTAAA----------------GTAGGGCCGGTAGTTGAATGCTTGCCTGTCAGACTGGATGACCAAGTTCAGTATCAACACAATATAGTGCCAGGAGCTAATTGTTCCCCAGCAGCGTGAC")
        check_component(a.components[1], "lav_tests.orange", 53, 252, "+", 361, "GTCCGGCCGGCTGTGTGCTACAATACACGTTCACGCAGTTTGGCCAATCACTTTAAGTATATACGAAATGGTTACCATGAGTTGTACTGTAAGGCAGCGGAAAGC---TTGTTAA--------CTCCTGGGCGACATT----GGGGCTGCAACATCGTTTATCCTCCTCTACAACCAATAGCTG-TTGCTTCTTGGTTCAAGTATATCCCATGGATTAGTATCAACACGATATAGTGTCAGGAGCTAATTGTTCCCCAGCAGCGTGAC")

        a = next(reader)
        assert a.score == 3586, "a.score is wrong: %s" % a.score
        assert len(a.components) == 2
        check_component(a.components[0], "apple", 52, 72, "+", 411, "TGCATATCGACTATTACAGCCACGCGAGTTACATTCCTCTTTTTTTTTGCTGGCGTCCGGCCGGCTGAGAGC")
        check_component(a.components[1], "lav_tests.orange", 2, 72, "-", 361, "TGCATATCGACTAGTACAGCCTCTCGAGTTACCCCCCCCATTCCTCTTGCTGACGTCACGCTGCTGGGGAAC")

        a = next(reader)
        assert a is None

        reader.close()


def check_component(c, src, start, size, strand, src_size, text):
    # ..print "\"%s\" == \"%s\"" % (c.src,src)
    assert c.src == src, "c.src = %s (expected %s)" % (c.src, src)
    assert c.start == start, "c.start = %s (expected %s)" % (c.start, start)
    assert c.size == size, "c.size = %s (expected %s)" % (c.size, size)
    assert c.strand == strand, "c.strand = %s (expected %s)" % (c.strand, strand)
    assert c.src_size == src_size, "c.src_size = %s (expected %s)" % (c.src_size, src_size)
    assert c.text == text, "c.text = \"%s\" (expected \"%s\")" % (c.text, text)



#!/usr/bin/env python

"""
Read a maf and print the text as a fasta file, concatenating blocks. A
specific subset of species can be chosen.

usage %prog [options] species1,species2,... < maf_file > fasta_file
    --fill="expression": Insert this between blocks
    --wrap=columns: Wrap FASTA to this many columns
"""
from __future__ import print_function

import sys
from optparse import OptionParser

from bx.align import maf


def __main__():
    # Parse command line arguments

    parser = OptionParser()
    parser.add_option("--fill", action="store", default=None, type="string", help="")
    parser.add_option("--wrap", action="store", default=None, type="int", help="")
    parser.add_option("--nowrap", action="store_true", default=False, dest="nowrap", help="")

    (options, args) = parser.parse_args()

    species = []
    for arg in args:
        species.extend(arg.split(','))

    fill = ""
    if options.fill:
        fill = eval(options.fill)

    wrap = 50
    if options.wrap is not None:
        wrap = options.wrap
    elif options.nowrap:
        wrap = 0

    # create the concatenated sequences

    texts = {}
    for s in species:
        texts[s] = []
    maf_reader = maf.Reader(sys.stdin)
    for m in maf_reader:
        for s in species:
            c = m.get_component_by_src_start(s)
            if c:
                texts[s].append(c.text)
            else:
                texts[s].append("-" * m.text_size)
    for s in species:
        print(">" + s)
        print_n(fill.join(texts[s]), wrap)


def print_n(s, n, f=sys.stdout):
    if n <= 0:
        print(s, file=f)
    else:
        p = 0
        while p < len(s):
            print(s[p:min(p+n, len(s))], file=f)
            p += n


if __name__ == "__main__":
    __main__()



"""
Classes that represent alignments between multiple sequences.
"""
import random
import string
import weakref

import six

from bx.misc.readlengths import read_lengths_file

# DNA reverse complement table
# DNA_COMP = "                                             -                  " \
#            " TVGH  CD  M KN   YSA BWXR       tvgh  cd  m kn   ysa bwxr      " \
#            "                                                                " \
#            "                                                                "

if six.PY2:
    DNA_COMP = string.maketrans("ACGTacgt", "TGCAtgca")
else:
    DNA_COMP = str.maketrans("ACGTacgt", "TGCAtgca")


class Alignment(object):

    def __init__(self, score=0, attributes={}, species_to_lengths=None):
        # species_to_lengths is needed only for file formats that don't provide
        # chromosome lengths;  it maps each species name to one of these:
        #   - the name of a file that contains a list of chromosome length pairs
        #   - a dict mapping chromosome names to their length
        #   - a single length value (useful when we just have one sequence and no chromosomes)
        # internally a file name is replaced by a dict, but only on an "as
        # needed" basis
        self.score = score
        self.text_size = 0
        self.attributes = attributes
        if species_to_lengths is None:
            self.species_to_lengths = {}
        else:
            self.species_to_lengths = species_to_lengths
        self.components = []

    def add_component(self, component):
        component._alignment = weakref.ref(self)
        self.components.append(component)
        if component.text is not None:
            if self.text_size == 0:
                self.text_size = len(component.text)
            elif self.text_size != len(component.text):
                raise Exception("Components must have same text length")

    def get_score(self):
        return self.__score

    def set_score(self, score):
        if isinstance(score, str):
            try:
                score = int(score)
            except ValueError:
                try:
                    score = float(score)
                except ValueError:
                    pass
        self.__score = score
    score = property(fget=get_score, fset=set_score)

    def __str__(self):
        s = "a score=" + str(self.score)
        for key in self.attributes:
            s += " %s=%s" % (key, self.attributes[key])
        s += "\n"
        # Components
        for c in self.components:
            s += str(c)
            s += "\n"
        return s

    def src_size(self, src):
        species, chrom = src_split(src)
        if species in self.species_to_lengths:
            chrom_to_length = self.species_to_lengths[species]
        elif chrom in self.species_to_lengths:
            chrom_to_length = self.species_to_lengths
        else:
            raise ValueError("no src_size (no length file for %s)" % species)
        if isinstance(chrom_to_length, int):         # (if it's a single length)
            return chrom_to_length
        if isinstance(chrom_to_length, str):  # (if it's a file name)
            chrom_to_length = read_lengths_file(chrom_to_length)
            self.species_to_lengths[species] = chrom_to_length
        if chrom not in chrom_to_length:
            raise ValueError("no src_size (%s has no length for %s)" % (species, chrom))
        return chrom_to_length[chrom]

    def get_component_by_src(self, src):
        for c in self.components:
            if c.src == src:
                return c
        return None

    def get_components_by_src(self, src):
        for c in self.components:
            if c.src == src:
                yield c

    def get_component_by_src_start(self, src):
        for c in self.components:
            if c.src.startswith(src):
                return c
        return None

    def slice(self, start, end):
        new = Alignment(score=self.score, attributes=self.attributes)
        for component in self.components:
            # FIXME: Is this the right solution?
            if component.empty:
                continue
            new.components.append(component.slice(start, end))
        new.text_size = end - start
        return new

    def reverse_complement(self):
        new = Alignment(score=self.score, attributes=self.attributes)
        for component in self.components:
            new.components.append(component.reverse_complement())
        new.text_size = self.text_size
        return new

    def slice_by_component(self, component_index, start, end):
        """
        Return a slice of the alignment, corresponding to an coordinate interval in a specific component.

        component_index is one of
            an integer offset into the components list
            a string indicating the src of the desired component
            a component

        start and end are relative to the + strand, regardless of the component's strand.

        """
        if isinstance(component_index, int):
            ref = self.components[component_index]
        elif isinstance(component_index, str):
            ref = self.get_component_by_src(component_index)
        elif isinstance(component_index, Component):
            ref = component_index
        else:
            raise ValueError("can't figure out what to do")
        start_col = ref.coord_to_col(start)
        end_col = ref.coord_to_col(end)
        if ref.strand == '-':
            (start_col, end_col) = (end_col, start_col)
        return self.slice(start_col, end_col)

    def column_iter(self):
        for i in range(self.text_size):
            yield [c.text[i] for c in self.components]

    def limit_to_species(self, species):
        new = Alignment(score=self.score, attributes=self.attributes)
        new.text_size = self.text_size
        for component in self.components:
            if component.src.split('.')[0] in species:
                new.add_component(component)
        return new

    def remove_all_gap_columns(self):
        """
        Remove any columns containing only gaps from alignment components,
        text of components is modified IN PLACE.
        """
        seqs = []
        for c in self.components:
            try:
                seqs.append(list(c.text))
            except TypeError:
                seqs.append(None)
        i = 0
        text_size = self.text_size
        while i < text_size:
            all_gap = True
            for seq in seqs:
                if seq is None:
                    continue
                if seq[i] != '-':
                    all_gap = False
            if all_gap:
                for seq in seqs:
                    if seq is None:
                        continue
                    del seq[i]
                text_size -= 1
            else:
                i += 1
        for i in range(len(self.components)):
            if seqs[i] is None:
                continue
            self.components[i].text = ''.join(seqs[i])
        self.text_size = text_size

    def __eq__(self, other):
        if other is None or not isinstance(other, type(self)):
            return False
        if self.score != other.score:
            return False
        if self.attributes != other.attributes:
            return False
        if len(self.components) != len(other.components):
            return False
        for c1, c2 in zip(self.components, other.components):
            if c1 != c2:
                return False
        return True

    def __ne__(self, other):
        return not(self.__eq__(other))

    def __deepcopy__(self, memo):
        from copy import deepcopy
        new = Alignment(score=self.score, attributes=deepcopy(self.attributes), species_to_lengths=deepcopy(self.species_to_lengths))
        for component in self.components:
            new.add_component(deepcopy(component))
        return new


class Component(object):

    def __init__(self, src='', start=0, size=0, strand=None, src_size=None, text=''):
        self._alignment = None
        self.src = src
        self.start = start          # Nota Bene:  start,size,strand are as they
        self.size = size            # .. appear in a MAF file-- origin-zero, end
        self.strand = strand        # .. excluded, and minus strand counts from
        self._src_size = src_size   # .. end of sequence
        self.text = text
        self.quality = None
        # Optional fields to keep track of synteny status (only makes sense
        # when the alignment is part of an ordered set)
        self.synteny_left = None
        self.synteny_right = None
        self.synteny_empty = None
        # If true, this component actually represents a non-aligning region,
        # and has no text.
        self.empty = False
        # Index maps a coordinate (distance along + strand from + start) to alignment column
        self.index = None

    def __str__(self):
        if self.empty:
            rval = "e %s %d %d %s %d %s" % (
                self.src, self.start, self.size, self.strand, self.src_size, self.synteny_empty)
        else:
            rval = "s %s %d %d %s %d %s" % (
                self.src, self.start, self.size, self.strand, self.src_size, self.text)
            if self.synteny_left and self.synteny_right:
                rval += "\ni %s %s %d %s %d" % (
                    self.src, self.synteny_left[0], self.synteny_left[1],
                    self.synteny_right[0], self.synteny_right[1])
        return rval

    def get_end(self):
        return self.start + self.size
    end = property(fget=get_end)

    def get_src_size(self):
        if self._src_size is None:
            if self._alignment is None:
                raise Exception("component has no src_size")
            self._src_size = self._alignment().src_size(self.src)
        return self._src_size

    def set_src_size(self, src_size):
        self._src_size = src_size
    src_size = property(fget=get_src_size, fset=set_src_size)

    def get_forward_strand_start(self):
        if self.strand == '-':
            return self.src_size - self.end
        else:
            return self.start
    forward_strand_start = property(fget=get_forward_strand_start)

    def get_forward_strand_end(self):
        if self.strand == '-':
            return self.src_size - self.start
        else:
            return self.end
    forward_strand_end = property(fget=get_forward_strand_end)

    def reverse_complement(self):
        start = self.src_size - self.end
        if self.strand == "+":
            strand = "-"
        else:
            strand = "+"
        comp = [ch for ch in self.text.translate(DNA_COMP)]
        comp.reverse()
        text = "".join(comp)
        new = Component(self.src, start, self.size, strand, self._src_size, text)
        new._alignment = self._alignment
        return new

    def slice(self, start, end):
        new = Component(src=self.src, start=self.start, strand=self.strand, src_size=self._src_size)
        new._alignment = self._alignment
        new.text = self.text[start:end]

        # for i in range( 0, start ):
        #    if self.text[i] != '-': new.start += 1
        # for c in new.text:
        #    if c != '-': new.size += 1
        new.start += start - self.text.count('-', 0, start)
        new.size = len(new.text) - new.text.count('-')

        # FIXME: This annotation probably means nothing after slicing if
        # one of the ends changes. In general the 'i' rows of a MAF only
        # make sense in context (relative to the previous and next alignments
        # in a stream, slicing breaks that).
        new.synteny_left = self.synteny_left
        new.synteny_right = self.synteny_right

        return new

    def slice_by_coord(self, start, end):
        """
        Return the slice of the component corresponding to a coordinate interval.

        start and end are relative to the + strand, regardless of the component's strand.

        """
        start_col = self.coord_to_col(start)
        end_col = self.coord_to_col(end)
        if (self.strand == '-'):
            (start_col, end_col) = (end_col, start_col)
        return self.slice(start_col, end_col)

    def coord_to_col(self, pos):
        """
        Return the alignment column index corresponding to coordinate pos.

        pos is relative to the + strand, regardless of the component's strand.

        """
        start, end = self.get_forward_strand_start(), self.get_forward_strand_end()
        if pos < start or pos > end:
            raise ValueError("Range error: %d not in %d-%d" % (pos, start, end))
        if not self.index:
            self.index = list()
            if self.strand == '-':
                # nota bene: for - strand self.index[x] maps to one column
                # higher than is actually associated with the position;  thus
                # when slice_by_component() and slice_by_coord() flip the ends,
                # the resulting slice is correct
                for x in range(len(self.text)-1, -1, -1):
                    if not self.text[x] == '-':
                        self.index.append(x + 1)
                self.index.append(0)
            else:
                for x in range(len(self.text)):
                    if not self.text[x] == '-':
                        self.index.append(x)
                self.index.append(len(self.text))
        x = None
        try:
            x = self.index[pos - start]
        except IndexError:
            raise Exception("Error in index.")
        return x

    def __eq__(self, other):
        if other is None or not isinstance(other, type(self)):
            return False
        return (self.src == other.src
                and self.start == other.start
                and self.size == other.size
                and self.strand == other.strand
                and self._src_size == other._src_size
                and self.text == other.text
                and self.synteny_left == other.synteny_left
                and self.synteny_right == other.synteny_right
                and self.synteny_empty == other.synteny_empty
                and self.empty == other.empty)

    def __ne__(self, other):
        return not(self.__eq__(other))

    def __deepcopy__(self, memo):
        new = Component(src=self.src, start=self.start, size=self.size, strand=self.strand, src_size=self._src_size, text=self.text)
        new._alignment = self._alignment
        new.quality = self.quality
        new.synteny_left = self.synteny_left
        new.synteny_right = self.synteny_right
        new.synteny_empty = self.synteny_empty
        new.empty = self.empty
        new.index = self.index
        return new


def get_reader(format, infile, species_to_lengths=None):
    import bx.align.axt
    import bx.align.lav
    import bx.align.maf
    if format == "maf":
        return bx.align.maf.Reader(infile, species_to_lengths)
    elif format == "axt":
        return bx.align.axt.Reader(infile, species_to_lengths)
    elif format == "lav":
        return bx.align.lav.Reader(infile)
    else:
        raise ValueError("Unknown alignment format %s" % format)


def get_writer(format, outfile, attributes={}):
    import bx.align.axt
    import bx.align.lav
    import bx.align.maf
    if format == "maf":
        return bx.align.maf.Writer(outfile, attributes)
    elif format == "axt":
        return bx.align.axt.Writer(outfile, attributes)
    elif format == "lav":
        return bx.align.lav.Writer(outfile, attributes)
    else:
        raise ValueError("Unknown alignment format %s" % format)


def get_indexed(format, filename, index_filename=None, keep_open=False, species_to_lengths=None):
    import bx.align.axt
    import bx.align.lav
    import bx.align.maf
    if format == "maf":
        return bx.align.maf.Indexed(filename, index_filename, keep_open, species_to_lengths)
    elif format == "axt":
        return bx.align.axt.Indexed(filename, index_filename, keep_open, species_to_lengths)
    elif format == "lav":
        raise Exception("LAV support for Indexed has not been implemented")
    else:
        raise ValueError("Unknown alignment format %s" % format)


def shuffle_columns(a):
    """Randomize the columns of an alignment"""
    mask = range(a.text_size)
    random.shuffle(mask)
    for c in a.components:
        c.text = ''.join([c.text[i] for i in mask])


def src_split(src):  # splits src into species,chrom
    dot = src.rfind(".")
    if dot == -1:
        return None, src
    else:
        return src[:dot], src[dot+1:]


def src_merge(species, chrom, contig=None):  # creates src (inverse of src_split)
    if species is None:
        src = chrom
    else:
        src = species + "." + chrom
    if contig is not None:
        src += "[%s]" % contig
    return src

# ---- Read C extension if available ---------------------------------------


try:
    from ._core import coord_to_col
except ImportError:
    def coord_to_col(start, text, pos):
        col = 0
        while start < pos:
            if text[col] != '-':
                start += 1
            col += 1
        return col



#!/usr/bin/env python

"""
Reads a list of intervals and a maf. Produces a new maf containing the
blocks or parts of blocks in the original that overlapped the intervals.

It is assumed that each file `maf_fname` has a corresponding `maf_fname`.index
file.

NOTE: If two intervals overlap the same block it will be written twice. With
      non-overlapping intervals and --chop this is never a problem.

NOTE: Intervals are origin-zero, half-open.  For example, the interval 100,150
      is 50 bases long, and there are 100 bases to its left in the sequence.

NOTE: Intervals are relative to the + strand, regardless of the strands in
      the alignments.


WARNING: bz2/bz2t support and file cache support are new and not as well
         tested.

usage: %prog maf_fname1 maf_fname2 ... [options] < interval_file
   -m, --mincols=0: Minimum length (columns) required for alignment to be output
   -c, --chop:       Should blocks be chopped to only portion overlapping (no by default)
   -s, --src=s:      Use this src for all intervals
   -p, --prefix=p:   Prepend this to each src before lookup
   -d, --dir=d:      Write each interval as a separate file in this directory
   -S, --strand:     Strand is included as an additional column, and the blocks are reverse complemented (if necessary) so that they are always on that strand w/r/t the src species.
   -C, --usecache:   Use a cache that keeps blocks of the MAF files in memory (requires ~20MB per MAF)
"""

import os
import sys

import bx.align.maf
from bx.cookbook import doc_optparse


def main():
    # Parse Command Line
    options, args = doc_optparse.parse(__doc__)
    try:
        maf_files = args
        if options.mincols:
            mincols = int(options.mincols)
        else:
            mincols = 0
        if options.src:
            fixed_src = options.src
        else:
            fixed_src = None
        if options.prefix:
            prefix = options.prefix
        else:
            prefix = None
        if options.dir:
            dir = options.dir
        else:
            dir = None
        chop = bool(options.chop)
        do_strand = bool(options.strand)
        use_cache = bool(options.usecache)
    except Exception:
        doc_optparse.exit()
    # Open indexed access to mafs
    index = bx.align.maf.MultiIndexed(maf_files, keep_open=True, parse_e_rows=True, use_cache=use_cache)
    # Start MAF on stdout
    if dir is None:
        out = bx.align.maf.Writer(sys.stdout)
    # Iterate over input ranges
    for line in sys.stdin:
        strand = None
        fields = line.split()
        if fixed_src:
            src, start, end = fixed_src, int(fields[0]), int(fields[1])
            if do_strand:
                strand = fields[2]
        else:
            src, start, end = fields[0], int(fields[1]), int(fields[2])
            if do_strand:
                strand = fields[3]
        if prefix:
            src = prefix + src
        # Find overlap with reference component
        blocks = index.get(src, start, end)
        # Open file if needed
        if dir:
            out = bx.align.maf.Writer(open(os.path.join(dir, "%s:%09d-%09d.maf" % (src, start, end)), 'w'))
        # Write each intersecting block
        if chop:
            for block in blocks:
                for ref in block.get_components_by_src(src):
                    slice_start = max(start, ref.get_forward_strand_start())
                    slice_end = min(end, ref.get_forward_strand_end())
                    if slice_end <= slice_start:
                        continue
                    sliced = block.slice_by_component(ref, slice_start, slice_end)
                    # If the block is shorter than the minimum allowed size, stop
                    if mincols and (sliced.text_size < mincols):
                        continue
                    # If the reference component is empty, don't write the block
                    if sliced.get_component_by_src(src).size < 1:
                        continue
                    # Keep only components that are not empty
                    sliced.components = [c for c in sliced.components if c.size > 0]
                    # Reverse complement if needed
                    if strand is not None and ref.strand != strand:
                        sliced = sliced.reverse_complement()
                    # Write the block
                    out.write(sliced)
        else:
            for block in blocks:
                out.write(block)
        if dir:
            out.close()
    # Close output MAF
    out.close()
    index.close()


if __name__ == "__main__":
    main()



#!/usr/bin/env python2.4
"""
Returns all positions of a maf with any pwm score > threshold
The positions are projected onto human coordinates
"""
from __future__ import print_function

import sys

import bx.pwm.position_weight_matrix as pwmx
from bx.align import maf as align_maf
from bx.pwm.pwm_score_maf import MafMotifSelect


def isnan(x):
    return not x == x


def main():

    if len(sys.argv) < 5:
        print("%s transfac|basic pwmfile inmaf threshold [motif]" % sys.argv[0], file=sys.stderr)
        sys.exit(2)

    r = pwmx.Reader(open(sys.argv[2]), format=sys.argv[1])
    pwm = next(iter(r))
    inmaf = open(sys.argv[3])
    threshold = float(sys.argv[4])
    if len(sys.argv) > 5:
        motif = sys.argv[5]
    else:
        motif = None

    for maf in align_maf.Reader(inmaf):
        for mafmotif, pwm_score, motif_score in MafMotifSelect(maf, pwm, motif, threshold):
            print(mafmotif, pwm_score, motif_score)
            print('zzzzzzzzzzzzzzzzzzzzzzzzzzzzz')


def mafwrite(alignment, kvec=None, jvec=None, file=sys.stdout):
    file.write("a score=" + str(alignment.score))
    for key in alignment.attributes:
        file.write(" %s=%s" % (key, alignment.attributes[key]))
    file.write("\n")
    rows = []
    if not kvec:
        kvec = [0 for c in alignment.components]
    if not jvec:
        jvec = [0 for c in alignment.components]
    for c, x, y in zip(alignment.components, kvec, jvec):
        rows.append(("s", c.src, str(c.start), str(c.size), c.strand, str(c.src_size), c.text, "%.2f" % x, str(y)))
        file.write(format_tabular(rows, "llrrrrrrr"))


def format_tabular(rows, align=None):
    if len(rows) == 0:
        return ""
    lengths = [len(col) for col in rows[0]]
    for row in rows[1:]:
        for i in range(0, len(row)):
            lengths[i] = max(lengths[i], len(row[i]))
    rval = ""
    for row in rows:
        for i in range(0, len(row)):
            if align and align[i] == "l":
                rval += row[i].ljust(lengths[i])
            else:
                rval += row[i].rjust(lengths[i])
            rval += " "
        rval += "\n"
    return rval


if __name__ == '__main__':
    main()



#!/usr/bin/env python2.3

"""
Read a MAF from standard input and print counts of alignments, bases, or
columns.

usage: %prog [options]
   -c, --cols: count alignment columns rather than number of alignments
   -b, --bases: count bases in first species rather than number of alignments
   -s, --skip=N: when counting bases, skip this base
   -e, --each: print a count for each alignment rather than whole file
   -r, --ref=N: reference sequence (first by default, 0..n)
"""
from __future__ import print_function

import sys

import bx.align.maf
from bx.cookbook import doc_optparse


def __main__():
    options, args = doc_optparse.parse(__doc__)

    try:
        if options.cols:
            action = "cols"
        elif options.bases:
            action = "bases"
        else:
            action = "aligns"
        print_each = bool(options.each)
        if options.ref:
            ref = int(options.ref)
        else:
            ref = 0
        if options.skip:
            skip = options.skip
        else:
            skip = None
    except Exception:
        doc_optparse.exit()

    maf_reader = bx.align.maf.Reader(sys.stdin)

    count = 0

    for m in maf_reader:

        if action == "aligns":
            count += 1
        elif action == "cols":
            count += m.text_size
        elif action == "bases":
            if skip:
                count += (m.components[ref].size - m.components[ref].text.count(skip))
            else:
                count += m.components[ref].size

        if print_each:
            print(count)
            count = 0

    if not print_each:
        print(count)


if __name__ == "__main__":
    __main__()



from __future__ import division

__all__ = [ 'ArrayTree', 'FileArrayTreeDict', 'array_tree_dict_from_reader' ]

import numpy
from numpy import *
cimport numpy

cimport bx.arrays.wiggle

from bx.misc.binary_file import BinaryFileWriter, BinaryFileReader
from bx.misc.cdb import FileCDBDict

"""
Classes for storing binary data on disk in a tree structure that allows for
efficient sparse storage (when the data occurs in contiguous blocks), fast
access to a specific block of data, and fast access to summaries at different
resolutions.

On disk format
--------------

Blocks are stored contiguously on disk in level-order. Contents should always be
network byte order (big endian), however this implementation will byte-swap when
reading  if necessary. File contents:

- magic:      uint32
- version:    unit32
- array size: uint32
- block size: uint32
- array type: 4 chars (numpy typecode, currently only simple types represented by one char are supported)

- Internal nodes in level order
    - Summary
        - count of valid values in each subtree : sizeof( dtype ) * block_size
        - frequencies: sizeof ( int32 ) * block_size
        - min of valid values in each subtree : sizeof( dtype ) * block_size
        - max of valid values in each subtree : sizeof( dtype ) * block_size
        - sum of valid values in each subtree : sizeof( dtype ) * block_size
        - sum of squares of valid values in each subtree : sizeof( dtype ) * block_size
    - File offsets of each child node: uint64 * block_size
  
- Leaf nodes
    - data points: sizeof( dtype ) * block_size
    
- Version 1 reads version 0 and version 1

"""

## Enhancement ideas:
## 
##   - Write markers of the number of blocks skipped between blocks. This would
##     allow fast striding across data or summaries (use the indexes to get to
##     the start of a block, then just read straight through). Would this help?
## 
##   - Compression for blocks?

MAGIC = 0x310ec7dc
VERSION = 1
NUM_SUMMARY_ARRAYS = 6

def array_tree_dict_from_reader( reader, sizes, default_size=2147483647, block_size=1000, no_leaves=False ):
    # Create empty array trees
    rval = {}
    ## for key, size in sizes.iteritems():
    ##    rval[ key ] = ArrayTree( size, 1000 )
    # Fill
    last_chrom = None
    last_array_tree = None
    for chrom, start, end, _, val in reader:
        if chrom != last_chrom:
            if chrom not in rval:
                rval[chrom] = ArrayTree( sizes.get( chrom, default_size ), block_size, no_leaves=no_leaves )
            last_array_tree = rval[chrom]
        last_array_tree.set_range( start, end, val )
    return rval
                

cdef class FileArrayTreeDict:
    """
    Access to a file containing multiple array trees indexed by a string key.
    """
    cdef object io
    cdef object cdb_dict
    def __init__( self, file ):
        self.io = io = BinaryFileReader( file, MAGIC )
        assert (0 <= io.read_uint32() <= 1) # Check for version 0 or 1
        self.cdb_dict = FileCDBDict( file, is_little_endian=io.is_little_endian )
    def __getitem__( self, key ):
        offset = self.cdb_dict[key]
        offset = self.io.unpack( "L", offset.encode() )[0]
        self.io.seek( offset )
        return FileArrayTree( self.io.file, self.io.is_little_endian )
    
    @classmethod
    def dict_to_file( Class, dict, file, is_little_endian=True, no_leaves=False ):
        """
        Writes a dictionary of array trees to a file that can then be
        read efficiently using this class.
        """
        io = BinaryFileWriter( file, is_little_endian=is_little_endian )
        # Write magic number and version
        io.write_uint32( MAGIC )
        io.write_uint32( VERSION )
        # Write cdb index with fake values just to fill space
        cdb_dict = {}
        for key in dict.iterkeys():
            cdb_dict[ key ] = io.pack( "L", 0 )
        cdb_offset = io.tell()
        FileCDBDict.to_file( cdb_dict, file, is_little_endian=is_little_endian )
        # Write each tree and save offset
        for key, value in dict.iteritems():
            offset = io.tell()
            cdb_dict[ key ] = io.pack( "L", offset )
            value.to_file( file, is_little_endian=is_little_endian, no_leaves=no_leaves )
        # Go back and write the index again
        io.seek( cdb_offset )
        FileCDBDict.to_file( cdb_dict, file, is_little_endian=is_little_endian )
        
cdef class FileArrayTree:
    """
    Wrapper for ArrayTree stored in file that reads as little as possible
    """
    cdef public int max
    cdef public int block_size
    cdef public object dtype
    cdef public int levels
    cdef public int offset
    cdef public int root_offset
    cdef object io
    
    def __init__( self, file, is_little_endian=True ):
        self.io = BinaryFileReader( file, is_little_endian=is_little_endian )
        self.offset = self.io.tell()
        # Read basic info about the tree
        self.max = self.io.read_uint32()
        self.block_size = self.io.read_uint32()
        # Read dtype and canonicalize
        dt = self.io.read( 1 )
        self.dtype = numpy.dtype( dt )
        self.io.skip( 3 )
        # How many levels are needed to cover the entire range?
        self.levels = 0
        while ( <long long> self.block_size ) ** ( self.levels + 1 ) < self.max:
            self.levels += 1
        # Not yet dealing with the case where the root is a Leaf
        assert self.levels > 0, "max < block_size not yet handled"
        # Save offset of root
        self.root_offset = self.io.tell()
        
    def __getitem__( self, index ):
        min = self.r_seek_to_node( index, 0, self.root_offset, self.levels, 0 )
        if min < 0:
            return nan
        self.io.skip( self.dtype.itemsize * ( index - min ) )
        return self.io.read_raw_array( self.dtype, 1 )[0]
        
    def get_summary( self, index, level ):
        if level <= 0 or level > self.levels:
            raise ValueError, "level must be <= self.levels"
        if self.r_seek_to_node( index, 0, self.root_offset, self.levels, level ) < 0:
            return None
        # Read summary arrays
        s = Summary()
        s.counts = self.io.read_raw_array( self.dtype, self.block_size )
        s.frequencies = self.io.read_raw_array( self.dtype, self.block_size )
        s.sums = self.io.read_raw_array( self.dtype, self.block_size )
        s.mins = self.io.read_raw_array( self.dtype, self.block_size)
        s.maxs = self.io.read_raw_array( self.dtype, self.block_size )
        s.sumsquares = self.io.read_raw_array( self.dtype, self.block_size )
        return s
        
    def get_leaf( self, index ):
        if self.r_seek_to_node( index, 0, self.root_offset, self.levels, 0 ) < 0:
            return []
        return self.io.read_raw_array( self.dtype, self.block_size )
        
    cdef int r_seek_to_node( self, int index, int min, long long offset, int level, int desired_level ):
        """
        Seek to the start of the node at `desired_level` that contains `index`.
        Returns the minimum value represented in that node.
        """
        cdef int child_size, bin_index, child_min
        self.io.seek( offset )
        if level > desired_level:
            child_size = self.block_size ** level
            bin_index = ( index - min ) // ( child_size ) 
            child_min = min + ( bin_index * child_size )
            # Skip summary arrays -- # arrays * itemsize * block_size
            self.io.skip( NUM_SUMMARY_ARRAYS * self.dtype.itemsize * self.block_size )
            # Skip to offset of correct child -- offsets are 8 bytes
            self.io.skip( 8 * bin_index )
            # Read offset of child
            child_offset = self.io.read_uint64()
            # print "co: %s\tbi: %s\tcm: %s\n" % (child_offset, bin_index, child_min)
            if child_offset == 0:
                return -1
            return self.r_seek_to_node( index, child_min, child_offset, level - 1, desired_level )
        else:
            # The file pointer is at the start of the desired node, do nothing
            return min

cdef class Summary:
    """
    Summary for a non-leaf level of the tree, contains arrays of the min, max,
    valid count, sum, and sum-of-squares for each child.
    """
    cdef public object counts
    cdef public object frequencies
    cdef public object mins
    cdef public object maxs
    cdef public object sums
    cdef public object sumsquares

cdef class ArrayTreeNode
cdef class ArrayTreeLeaf

cdef class ArrayTree:
    """
    Stores a sparse array of data as a tree.
    
    An array of `self.max` values is stored in a tree in which each leaf
    contains `self.block_size` values and each internal node contains
    `self.block_size` children.
    
    Entirely empty subtrees are not stored. Thus, the storage is efficient for
    data that is block sparse -- having contiguous chunks of `self.block_size` or
    larger data. Currently it is not efficient if the data is strided (e.g.
    one or two data points in every interval of length `self.block_size`).
    
    Internal nodes store `Summary` instances for their subtrees. 
    """

    cdef public int max
    cdef public int block_size
    cdef public object dtype
    cdef public int levels
    cdef public int no_leaves
    cdef public ArrayTreeNode root

    def __init__( self, int max, int block_size, dtype=float32, no_leaves=False ):
        """
        Create a new array tree of size `max` 
        """
        self.max = max
        self.block_size = block_size
        self.no_leaves = no_leaves
        # Force the dtype argument to its canonical dtype object
        self.dtype = numpy.dtype( dtype )
        # How many levels are needed to cover the entire range?
        self.levels = 0
        while ( <long long> self.block_size ) ** ( self.levels + 1 ) < self.max:
            self.levels += 1
        # Not yet dealing with the case where the root is a Leaf
        assert self.levels > 0, "max < block_size not yet handled"
        # Create the root node`
        self.root = ArrayTreeNode( self, 0, max, block_size, self.levels )
        
    def __setitem__( self, int index, value ):
        self.root.set( index, value )
        
    def set_range( self, int start, int end, value ):
        for i from start <= i < end:
            self.root.set( i, value )
        
    def __getitem__( self, int index ):
        return self.root.get( index )
        
    def to_file( self, f, is_little_endian=True, no_leaves=False ):
        io = BinaryFileWriter( f, is_little_endian=is_little_endian )
        ## io.write_uint32( VERSION )
        io.write_uint32( self.max )
        io.write_uint32( self.block_size )
        io.write( self.dtype.char )
        io.write( "\0\0\0" )
        # Data pass, level order
        if no_leaves:
            bottom_level = 0
        else:
            bottom_level = -1
        for level in range( self.levels, bottom_level, -1 ):
            self.root.to_file_data_pass( io, level )
        # Offset pass to fix up indexes
        self.root.to_file_offset_pass( io )
        
    @classmethod
    def from_file( Class, f, is_little_endian=True ):        
        io = BinaryFileReader( f, is_little_endian=is_little_endian )
        ## assert io.read_uint32() == VERSION
        max = io.read_uint32()
        block_size = io.read_uint32()
        dt = io.read( 1 )
        io.read( 3 )
        tree = Class( max, block_size, dt )
        tree.root.from_file( io )
        return tree
    
    @classmethod
    def from_sequence( Class, s, block_size=1000 ):
        """
        Build an ArrayTree from a sequence like object (must have at least
        length and getitem).
        """
        tree = Class( len( s ), block_size )
        for i in range( len( s ) ):
            tree[i] = s[i]
        return tree
        
cdef class ArrayTreeNode:
    """
    Internal node of an ArrayTree. Contains summary data and pointers to
    subtrees.
    """

    cdef ArrayTree tree
    cdef int min
    cdef int max
    cdef int block_size
    cdef int level
    cdef int child_size
    cdef object children
    cdef public Summary summary
    cdef public long start_offset

    def __init__( self, ArrayTree tree, int min, int max, int block_size, int level ):
        self.tree = tree
        self.min = min
        self.max = max
        self.block_size = block_size
        self.level = level
        # Each of my children represents block_size ** level values
        self.child_size = self.block_size ** self.level        
        self.children = [None] * self.block_size
        self.summary = None
        self.start_offset = 0
        
    cdef inline init_bin( self, int index ):
        cdef int min = self.min + ( index * self.child_size )
        cdef int max = min + self.child_size
        if self.level == 1:
            self.children[ index ] = ArrayTreeLeaf( self.tree, min, max )
        else:
            self.children[ index ] = ArrayTreeNode( self.tree, min, max, self.block_size, self.level - 1 )
            
    def set( self, int index, value ):
        cdef int bin_index = ( index - self.min ) // ( self.child_size )
        if self.children[ bin_index ] is None:
            self.init_bin( bin_index )
        self.children[ bin_index ].set( index, value )
        
    def get( self, int index ):
        cdef int bin_index = ( index - self.min ) // ( self.child_size ) 
        if self.children[ bin_index ] is None:
            return nan
        else:
            return self.children[ bin_index ].get( index )
            
    cpdef build_summary( self ):
        """
        Build summary of children. 
        """
        counts = empty( self.tree.block_size, self.tree.dtype )
        frequencies = empty( self.tree.block_size, self.tree.dtype )
        mins = empty( self.tree.block_size, self.tree.dtype )
        maxs = empty( self.tree.block_size, self.tree.dtype )
        sums = empty( self.tree.block_size, self.tree.dtype )
        sumsquares = empty( self.tree.block_size, self.tree.dtype )
        for i in range( len( self.children ) ):
            if self.children[i]:
                if self.level == 1:
                    v = self.children[i].values
                    counts[i] = sum( ~isnan( v ) )
                    frequencies[i] = self.children[i].frequency
                    mins[i] = nanmin( v )
                    maxs[i] = nanmax( v )
                    sums[i] = nansum( v )
                    sumsquares[i] = nansum( v ** 2 )
                else:
                    c = self.children[i]
                    c.build_summary()
                    counts[i] = sum( c.summary.counts )
                    frequencies[i] = sum( c.summary.frequencies )
                    mins[i] = nanmin( c.summary.mins )
                    maxs[i] = nanmax( c.summary.maxs )
                    sums[i] = nansum( c.summary.sums )
                    sumsquares[i] = nansum( c.summary.sumsquares )
            else:
                counts[i] = 0
                frequencies[i] = 0
                mins[i] = nan
                maxs[i] = nan
                sums[i] = nan
                sumsquares[i] = nan
        s = Summary()
        s.counts = counts
        s.frequencies = frequencies
        s.mins = mins
        s.maxs = maxs
        s.sums = sums
        s.sumsquares = sumsquares
        self.summary = s
        
    def to_file_data_pass( self, io, level ):
        """
        First pass of writing to file, writes data and saves position of block.
        """
        assert self.summary, "Writing without summaries is currently not supported"
        # If we are at the current level being written, write a block
        if self.level == level:
            # Save file offset where this block starts
            self.start_offset = io.tell()
            # Write out summary data
            io.write_raw_array( self.summary.counts )
            io.write_raw_array( self.summary.frequencies )
            io.write_raw_array( self.summary.sums )
            io.write_raw_array( self.summary.mins )
            io.write_raw_array( self.summary.maxs )
            io.write_raw_array( self.summary.sumsquares )
            # Skip enough room for child offsets (block_size children * 64bits)
            io.skip( self.tree.block_size * 8 )
        # Must be writing a lower level, so recurse
        else:
            # Write all non-empty children
            for i in range( len( self.children ) ):
                if self.children[i] is not None:
                    self.children[i].to_file_data_pass( io, level )
                
    def to_file_offset_pass( self, io ):
        """
        Second pass of writing to file, seek to appropriate position and write
        offsets of children.
        """
        # Seek to location of child offfsets (skip over # summary arrays)
        skip_amount = NUM_SUMMARY_ARRAYS * self.tree.dtype.itemsize * self.block_size
        io.seek( self.start_offset + skip_amount )
        # Write the file offset of each child into the index
        for child in self.children:
            if child is None:
                io.write_uint64( 0 )
            else:
                io.write_uint64( child.start_offset )
        # Recursively write offsets in child nodes
        for child in self.children:
            if child is not None:
                child.to_file_offset_pass( io )
    
    def from_file( self, io ):
        """
        Load entire summary and all children into memory.
        """
        dtype = self.tree.dtype
        block_size = self.tree.block_size
        # Read summary arrays
        s = Summary()
        s.counts = io.read_raw_array( dtype, block_size )
        s.frequencies = io.read_raw_array( int32, block_size )
        s.sums = io.read_raw_array( dtype, block_size )
        s.mins = io.read_raw_array( dtype, block_size)
        s.maxs = io.read_raw_array( dtype, block_size )
        s.sumsquares = io.read_raw_array( dtype, block_size )
        self.summary = s
        # Read offset of all children
        child_offsets = [ io.read_uint64() for i in range( block_size ) ]
        for i in range( block_size ):
            if child_offsets[i] > 0:
                self.init_bin( i )
                io.seek( child_offsets[i] )
                self.children[i].from_file( io )
                
    def get_from_file( self, io, index ):
        cdef int bin_index = ( index - self.min ) //( self.child_size ) 
        if self.children[ bin_index ] is None:
            return nan
        else:
            return self.children[ bin_index ].get( index )
        
cdef class ArrayTreeLeaf:
    """
    Leaf node of an ArrayTree, contains data values.
    """
    
    cdef ArrayTree tree
    cdef int min
    cdef int max
    cdef public int frequency
    cdef public numpy.ndarray values
    cdef public long start_offset
    
    def __init__( self, ArrayTree tree, int min, int max ):
        self.tree = tree
        self.min = min
        self.max = max
        self.frequency = 0
        self.values = empty( max - min, self.tree.dtype )
        self.values[:] = nan
        self.start_offset = 0
        
    def set( self, index, value ):
        self.frequency += 1
        self.values[ index - self.min ] = value
        
    def get( self, index ):
        return self.values[ index - self.min ]
        
    def to_file_data_pass( self, io, level ):
        assert level == 0
        self.start_offset = io.tell()
        io.write_raw_array( self.values )
    
    def to_file_offset_pass( self, io ):
        pass
        
    def from_file( self, io ):
        self.values = io.read_raw_array( self.tree.dtype, self.tree.block_size )



from numpy import allclose
from six import StringIO

from . import transfac

sample = """
VV  TRANSFAC MATRIX TABLE, Rel.3.2 26-06-1997
XX
//
AC  a
XX
ID  V$MYOD_01
XX
DT  19.10.92 (created); ewi.
DT  16.10.95 (updated); ewi.
XX
NA  MyoD
XX
DE  myoblast determination gene product
XX
BF  T00526; MyoD; Species: mouse, Mus musculus.
XX
P0      A      C      G      T
01     100     200     200     0     S
02     200     100     200     0     R
03     300     0     100     100     A
04     0     500     0     0     C
05     500     0     0     0     A
06     0     0     400     100     G
07     0     100     400     0     G
08     0     0     0     500     T
09     0     0     500     0     G
10     0     100     200     200     K
11     0     200     0     300     Y
12     100     0     300     100     G
XX
BA  5 functional elements in 3 genes
XX
//
AC  M00002
XX
ID  V$E47_01
XX
DT  19.10.92 (created); ewi.
DT  16.10.95 (updated); ewi.
XX
NA  E47
XX
DE  E47
XX
BF  T00207; E47; Species: human, Homo sapiens.
XX
P0      A      C      G      T
00     400     400     300     0     N
02     200     500     400     0     S
03     300     200     400     200     N
04     200     0     900     0     G
05     0     1100     0     0     C
06     1100     0     0     0     A
07     0     0     1100     0     G
08     100     200     800     0     G
09     0     0     0     1100     T
10     0     0     1100     0     G
11     0     0     400     700     K
12     100     400     300     300     N
13     100     600     200     200     C
14     100     400     400     200     N
15     100     400     200     300     N
XX
BA  11 selected strong binding sites for E47, E47-MyoD, E12+MyoD 
BA  and (weak) for E12
XX
CC  Group I in [903]; 5 sites selected in vitro for binding to E12N 
CC  (=N-terminally truncated E12); matrix corrected according to 
CC  the published sequences
XX
RN  [1]
RA  Sun X.-H., Baltimore D.
RT  An inhibitory domain of E12 transcription factor prevents 
RT  DNA binding in E12 homodimers but not in E12 heterodimers
RL  Cell 64:459-470 (1991).
XX
"""  # noqa: W291


def test_reader():
    input = StringIO(sample)
    motifs = list(transfac.TransfacReader(input))
    assert len(motifs) == 2
    # Single value parse
    assert motifs[1].accession == "M00002"
    # Value list parse
    assert motifs[1].dates == ['19.10.92 (created); ewi.', '16.10.95 (updated); ewi.']
    # Matrix parse
    assert motifs[1].matrix.sorted_alphabet == ['A', 'C', 'G', 'T']
    assert allclose(motifs[1].matrix.values[0], [400, 400, 300, 0])



#!/usr/bin/env python

"""
Build an index file for a set of MAF alignment blocks.

If index_file is not provided maf_file.index is used.

usage: %prog maf_file index_file
    -s, --species=a,b,c: only index the position of the block in the listed species
"""

import os.path
from io import TextIOWrapper

from six import PY3

import bx.align.maf
from bx import interval_index_file
from bx.cookbook import doc_optparse
from bx.misc.seekbzip2 import SeekableBzip2File


def main():
    options, args = doc_optparse.parse(__doc__)

    try:
        maf_file = args[0]
        # If it appears to be a bz2 file, attempt to open with table
        if maf_file.endswith(".bz2"):
            table_file = maf_file + "t"
            if not os.path.exists(table_file):
                doc_optparse.exit("To index bz2 compressed files first "
                                  "create a bz2t file with bzip-table.")
            # Open with SeekableBzip2File so we have tell support
            maf_in = SeekableBzip2File(maf_file, table_file)
            # Strip .bz2 from the filename before adding ".index"
            maf_file = maf_file[:-4]
        elif maf_file.endswith(".lzo"):
            from bx.misc.seeklzop import SeekableLzopFile
            table_file = maf_file + "t"
            if not os.path.exists(table_file):
                doc_optparse.exit("To index lzo compressed files first "
                                  "create a lzot file with lzop_build_offset_table.")
            # Open with SeekableBzip2File so we have tell support
            maf_in = SeekableLzopFile(maf_file, table_file)
            # Strip .lzo from the filename before adding ".index"
            maf_file = maf_file[:-4]
        else:
            maf_in = open(maf_file, "rb")
        # Determine the name of the index file
        if len(args) > 1:
            index_file = args[1]
        else:
            index_file = maf_file + ".index"
        if options.species:
            species = options.species.split(",")
        else:
            species = None
    except Exception:
        doc_optparse.exception()

    if PY3:
        maf_in = TextIOWrapper(maf_in, encoding="ascii")
    maf_reader = bx.align.maf.Reader(maf_in)

    indexes = interval_index_file.Indexes()

    # Need to be a bit tricky in our iteration here to get the 'tells' right
    while True:
        pos = maf_reader.file.tell()
        block = next(maf_reader)
        if block is None:
            break
        for c in block.components:
            if species is not None and c.src.split('.')[0] not in species:
                continue
            indexes.add(c.src, c.forward_strand_start, c.forward_strand_end, pos, max=c.src_size)

    out = open(index_file, 'wb')
    indexes.write(out)
    out.close()


if __name__ == "__main__":
    main()



"""
Rudimentary support for PHAST's tree model file format (a simple format for
storing trees and rate matrices).
"""

from numpy import zeros


class TreeModel(object):
    def __init__(self):
        self.alphabet = None
        self.radix = 0
        self.order = 0
        self.subst_mod = None
        self.background = None
        self.tree = None
        self.matrix = None

    @staticmethod
    def from_file(f):
        input = iter(f)
        tm = TreeModel()
        for line in input:
            if line.startswith("ALPHABET:"):
                tm.alphabet = tuple(line.split()[1:])
                tm.radix = len(tm.alphabet)
            if line.startswith("ORDER:"):
                tm.order = int(line.split()[1])
            if line.startswith("SUBST_MOD:"):
                tm.subst_mod = line[11:].rstrip()
            if line.startswith("BACKGROUND:"):
                tm.background = tuple(map(float, line.split()[1:]))
            if line.startswith("TREE:"):
                tm.tree = line[6:].strip()
            if line.startswith("RATE_MAT:"):
                matrix = zeros((tm.radix, tm.radix), float)
                for i in range(len(tm.alphabet)):
                    matrix[i] = [float(_) for _ in next(input).split()]
                tm.matrix = matrix
        return tm



from tempfile import NamedTemporaryFile

from bx.misc.cdb import FileCDBDict


def test():

    d = {}
    for i in range(10000):
        d['foo' + str(i)] = 'bar' + str(i)

    # Open temporary file and get name
    file = NamedTemporaryFile()
    file_name = file.name

    # Write cdb to file
    FileCDBDict.to_file(d, file)
    file.flush()

    # Open on disk
    file2 = open(file_name, 'rb')
    cdb = FileCDBDict(file2)

    for key, value in d.items():
        assert cdb[key] == value

    try:
        cdb['notin']
        assert False, "KeyError was not raised"
    except KeyError:
        pass

    # Close everything (deletes the temporary file)
    file2.close()
    file.close()



"""
Support for dealing with (genome scale) sequence alignments. See `core` for
the abstract alignment classes and `maf`, `axt`, and `lav` for readers and
writers in various formats.
"""

from bx.align.core import *  # noqa



#!/usr/bin/env python2.3

"""
Read TFLOC output from stdin and write out a summary in which the nth line
contains the number of sites found in the nth alignment of the input.

TODO: This is very special case, should it be here?
"""
from __future__ import print_function

import sys
from collections import defaultdict

counts = defaultdict(int)

max_index = -1

for line in sys.stdin:
    if line[0].isdigit():
        current_index = int(line)
        max_index = max(current_index, max_index)
    elif line[0] == "'":
        counts[current_index] += 1
    else:
        raise ValueError("Invalid input line " + line)

for i in range(max_index + 1):
    print(counts.get(i, 0))



#!/usr/bin/env python

"""
Print the number of bases in a nib file.

usage: %prog nib_file
"""
from __future__ import print_function

import sys

from bx.seq import nib as seq_nib

with open(sys.argv[1]) as f:
    nib = seq_nib.NibFile(f)
print(nib.length)



#!/usr/bin/env python2.4

"""
FIXME!

usage: %prog feature.bed ar.bed snp.bed div_directory [options]
    -m, --mask=M: Mask AR and features with this file
    -s, --suffix=S: append suffix to chromosomes to get filenames from div_directory
    -l, --lens=l: Set chromosome ends using LEN file
"""
from __future__ import print_function

import sys

from bx.bitset import BinnedBitSet
from bx.bitset_builders import binned_bitsets_from_file
from bx.cookbook import doc_optparse


def main():
    options, args = doc_optparse.parse(__doc__)
    try:
        lens = {}
        if options.lens:
            for line in open(options.lens):
                chrom, length = line.split()
                lens[chrom] = int(length)

        if options.suffix:
            suffix = options.suffix
        else:
            suffix = ""

        print("\nReading feature", end=' ', file=sys.stderr)
        interval_file = open(args[0])
        feature = binned_bitsets_from_file(interval_file, lens=lens)
        interval_file.close()
        # reuse interval file
        intervals = {}
        interval_file = open(args[0])
        for line in interval_file:
            fields = line.split()
            chrom, start, end = fields[0], int(fields[1]), int(fields[2])
            if chrom not in intervals:
                intervals[chrom] = []
            intervals[chrom].append([start, end])
        interval_file.close()

        print("\nReading ar", end=' ', file=sys.stderr)
        ar = binned_bitsets_from_file(open(args[1]), lens=lens)

        print("\nReading snps", end=' ', file=sys.stderr)
        snp = binned_bitsets_from_file(open(args[2]), lens=lens)
        snp_mask = clone_inverted(snp)
        snp_copy = clone(snp)

        print("\nMasking AR", end=' ', file=sys.stderr)
        ar_mask = clone_inverted(ar)
        print(file=sys.stderr)

        dirname = args[3]

        if options.mask:
            mask = binned_bitsets_from_file(open(options.mask), lens=lens)
        else:
            mask = None
    except Exception:
        doc_optparse.exit()

    if mask:
        for chrom in mask.keys():
            if chrom in feature:
                feature[chrom].iand(mask[chrom])
            if chrom in ar:
                ar[chrom].iand(mask[chrom])

    # divergence and snp counts for all features
    feature_div_count = 0
    feature_snp_count = 0
    ar_div_count = 0
    ar_snp_count = 0

    # collect snp and div
    for chr in feature.keys():

        if chr not in snp:
            continue
        if chr not in ar:
            continue

        print("reading %s ..." % chr, end=' ', file=sys.stderr)
        try:
            div = binned_bitsets_from_file(open(dirname + "/%s.bed" % (chr+suffix)), lens=lens)
        except Exception:
            print("%s.bed not found" % chr, file=sys.stderr)
            continue

        div[chr].iand(snp_mask[chr])  # div/snp sites count snp-only
        div_copy = clone(div)

        print("AR:", chr, end=' ', file=sys.stderr)
        snp[chr].iand(ar[chr])
        div[chr].iand(ar[chr])
        snp_count = snp[chr].count_range(0, snp[chr].size)
        ar_snp_count += snp_count
        print(snp_count, end=' ', file=sys.stderr)
        try:
            div_count = div[chr].count_range(0, div[chr].size)
            ar_div_count += div_count
            print(div_count, file=sys.stderr)
        except Exception:
            print(chr, "failed", file=sys.stderr)

        div = div_copy
        snp[chr] = snp_copy[chr]
        print("feature:", chr, end=' ', file=sys.stderr)
        feature[chr].iand(ar_mask[chr])  # clip to non-AR only
        snp[chr].iand(feature[chr])
        div[chr].iand(feature[chr])
        feature_snp_count += snp[chr].count_range(0, snp[chr].size)
        print(snp[chr].count_range(0, snp[chr].size), div[chr].count_range(0, div[chr].size), file=sys.stderr)
        feature_div_count += div[chr].count_range(0, div[chr].size)
        print(snp[chr].count_range(0, snp[chr].size), div[chr].count_range(0, div[chr].size), file=sys.stderr)

        # Note: can loop over feature intervals here for individual counts
        if chr in intervals:
            for start, end in intervals[chr]:
                ind_div_count = div[chr].count_range(start, end-start)
                ind_snp_count = snp[chr].count_range(start, end-start)
                print(chr, start, end, ind_div_count, ind_snp_count)

    print("feature snp\t%d" % feature_snp_count)
    print("feature div\t%d" % feature_div_count)
    print("ar snp\t%d" % ar_snp_count)
    print("ar div\t%d" % ar_div_count)


# copies a dictionary of bitsets
def copybits(binnedbits):
    bitset = BinnedBitSet(binnedbits.size)
    bitset.ior(binnedbits)
    return bitset


def clone(bitsets):
    r = {}
    for k, b in bitsets.items():
        r[k] = copybits(b)
    return r


def clone_inverted(bitsets):
    r = {}
    for k, b in bitsets.items():
        r[k] = copybits(b)
        r[k].invert()
    return r


main()



#!/usr/bin/env python

"""
Read an alignment from stdin and for each block print the result of
evaluating `template_string` (in cheetah template format). The alignment
block will be placed in the template context as `a` and the list of components
as `c`.

usage: %prog template [options]
    -f, --format = maf: Input format, maf (default) or axt
"""

from __future__ import division, print_function


import sys

from bx import align
from bx.cookbook import doc_optparse

try:
    from Cheetah.Template import Template
except ImportError:
    print("This script requires the Cheetah template modules", file=sys.stderr)
    sys.exit(-1)


def main():

    # Parse command line arguments
    options, args = doc_optparse.parse(__doc__)

    try:
        template = Template(args[0])
        format = options.format
        if not format:
            format = "maf"
    except Exception:
        doc_optparse.exception()

    reader = align.get_reader(format, sys.stdin)

    for a in reader:
        template.a = a
        template.c = a.components
        print(template)


if __name__ == "__main__":
    main()



from bx.misc.binary_file import BinaryFileReader

DEF cir_tree_sig = 0x2468ACE0

cdef int ovcmp( bits32 a_hi, bits32 a_lo, bits32 b_hi, bits32 b_lo ):
    if a_hi < b_hi: 
        return 1
    elif a_hi > b_hi:
        return -1
    else:
        if a_lo < b_lo:
            return 1
        elif a_lo > b_lo:
            return -1
        else:
            return 0

cdef overlaps( qchrom, qstart, qend, rstartchrom, rstartbase, rendchrom, rendbase ):
    return ( ovcmp( qchrom, qstart, rendchrom, rendbase ) > 0 ) and \
           ( ovcmp( qchrom, qend, rstartchrom, rstartbase ) < 0 )

cdef class CIRTreeFile:

    def __init__( self, file=None ):
        if file is not None:
            self.attach( file )

    def attach( self, file ):
        """
        Attach to an open file
        """
        self.file = file
        self.reader = reader = BinaryFileReader( file, cir_tree_sig )
        self.is_byteswapped = self.reader.byteswap_needed
        # Header
        self.block_size = reader.read_uint32()
        self.item_count = reader.read_uint64()
        self.start_chrom_ix  = reader.read_uint32()
        self.start_base = reader.read_uint32()
        self.end_chrom_ix = reader.read_uint32()
        self.end_base = reader.read_uint32()
        self.file_size = reader.read_uint64()
        self.items_per_slot = reader.read_uint32()
        # Skip reserved
        reader.read_uint32()
        # Save root
        self.root_offset = reader.tell()

    def r_find_overlapping( self, int level, bits64 index_file_offset, bits32 chrom_ix, bits32 start, bits32 end, object rval, object reader ):
        cdef UBYTE is_leaf
        cdef bits16 child_count
        reader.seek( index_file_offset )
        # Block header
        is_leaf = reader.read_uint8()
        assert is_leaf == 0 or is_leaf == 1
        reader.read_uint8()
        child_count = reader.read_uint16()
        # Read block
        if is_leaf:
            self.r_find_overlapping_leaf( level, chrom_ix, start, end, rval, child_count, reader )
        else:
            self.r_find_overlapping_parent( level, chrom_ix, start, end, rval, child_count, reader )

    def r_find_overlapping_leaf( self, int level, bits32 chrom_ix, bits32 start, bits32 end, object rval, 
                                bits16 child_count, object reader ):
        cdef bits32 start_chrom_ix, start_base, end_chrom_ix, end_base
        cdef bits64 offset
        cdef bits64 size
        for i from 0 <= i < child_count:
            start_chrom_ix = reader.read_uint32()
            start_base = reader.read_uint32()
            end_chrom_ix = reader.read_uint32()
            end_base = reader.read_uint32()
            offset = reader.read_uint64()
            size = reader.read_uint64()
            if overlaps( chrom_ix, start, end, start_chrom_ix, start_base, end_chrom_ix, end_base ):
                rval.append( ( offset, size ) )

    def r_find_overlapping_parent( self, int level, bits32 chrom_ix, bits32 start, bits32 end, object rval, 
                                  bits16 child_count, object reader ):
        # Read and cache offsets for all children to avoid excessive seeking
        ## cdef bits32 start_chrom_ix[child_count], start_base[child_count], end_chrom_ix[child_count], end_base[child_count]
        ## cdef bits64 offset[child_count]
        start_chrom_ix = []; start_base = []; end_chrom_ix = []; end_base = []
        offset = []
        for i from 0 <= i < child_count:
            ## start_chrom_ix[i] = reader.read_bits32()
            ## start_base[i] = reader.read_bits32()
            ## end_chrom_ix[i] = reader.read_bits32()
            ## end_base[i] = reader.read_bits32()
            ## offset[i] = reader.read_bits64()
            start_chrom_ix.append( reader.read_uint32() )
            start_base.append( reader.read_uint32() )
            end_chrom_ix.append( reader.read_uint32() )
            end_base.append( reader.read_uint32() )
            offset.append( reader.read_uint64() )
        # Now recurse
        for i from 0 <= i < child_count:
            if overlaps( chrom_ix, start, end, start_chrom_ix[i], start_base[i], end_chrom_ix[i], end_base[i] ):
                self.r_find_overlapping( level + 1, offset[i], chrom_ix, start, end, rval, reader )

    def find_overlapping_blocks( self, bits32 chrom_ix, bits32 start, bits32 end ):
        rval = []
        self.r_find_overlapping( 0, self.root_offset, chrom_ix, start, end, rval, self.reader )
        return rval



"""
Classes for reading and writing motif data.
"""
from __future__ import print_function

from six import Iterator

from bx.motif.pwm import FrequencyMatrix


class TransfacMotif(object):

    def __init__(self):
        self.accession = None
        self.id = None
        self.dates = None
        self.name = None
        self.description = None
        self.binding_factors = None
        self.basis = None
        self.comment = None
        self.matrix = None
        self.attributes = None
        self.sites = None


transfac_actions = {
    "AC": ("store_single", "accession"),
    "ID": ("store_single", "id"),
    "DT": ("store_single_list", "dates"),
    "NA": ("store_single", "name"),
    "DE": ("store_block", "description"),
    "BF": ("store_single_list", "binding_factors"),
    "BA": ("store_block", "basis"),
    "CC": ("store_block", "comment"),
    "P0": ("store_matrix", "matrix"),
    # For CREAD format files
    "TY": ("store_single", "type"),
    "AT": ("store_single_key_value", "attributes"),
    "BS": ("store_single_list", "sites")
}


class TransfacReader(Iterator):
    """
    Reads motifs in TRANSFAC format.
    """

    parse_actions = transfac_actions

    def __init__(self, input):
        self.input = iter(input)
        self.input_exhausted = False

    def as_dict(self, key="id"):
        """
        Return a dictionary containing all remaining motifs, using `key`
        as the dictionary key.
        """
        rval = {}
        for motif in self:
            rval[getattr(motif, key)] = motif
        return rval

    def __iter__(self):
        return self

    def __next__(self):
        rval = self.next_motif()
        while rval is None:
            rval = self.next_motif()
        return rval

    def next_motif(self):
        if self.input_exhausted:
            raise StopIteration
        # Accumulate lines until either the end of record indicator "//" is
        # encounted or the input is exhausted.
        lines = []
        while True:
            try:
                line = next(self.input)
            except StopIteration:
                self.input_exhausted = True
                break
            if line.startswith("//"):
                break
            if not line.isspace():
                lines.append(line)
        if lines:
            return self.parse_record(lines)

    def parse_record(self, lines):
        """
        Parse a TRANSFAC record out of `lines` and return a motif.
        """
        # Break lines up
        temp_lines = []
        for line in lines:
            fields = line.rstrip("\r\n").split(None, 1)
            if len(fields) == 1:
                fields.append("")
            temp_lines.append(fields)
        lines = temp_lines
        # Fill in motif from lines
        motif = TransfacMotif()
        current_line = 0
        while True:
            # Done parsing if no more lines to consume
            if current_line >= len(lines):
                break
            # Remove prefix and first separator from line
            prefix, rest = lines[current_line]
            # No action for this prefix, just ignore the line
            if prefix not in self.parse_actions:
                current_line += 1
                continue
            # Get action for line
            action = self.parse_actions[prefix]
            # Store a single line value
            if action[0] == "store_single":
                key = action[1]
                setattr(motif, key, rest)
                current_line += 1
            # Add a single line value to a list
            if action[0] == "store_single_list":
                key = action[1]
                if not getattr(motif, key):
                    setattr(motif, key, [])
                getattr(motif, key).append(rest)
                current_line += 1
            # Add a single line value to a dictionary
            if action[0] == "store_single_key_value":
                key = action[1]
                k, v = rest.strip().split('=', 1)
                if not getattr(motif, key):
                    setattr(motif, key, {})
                getattr(motif, key)[k] = v
                current_line += 1
            # Store a block of text
            if action[0] == "store_block":
                key = action[1]
                value = []
                while current_line < len(lines) and lines[current_line][0] == prefix:
                    value.append(lines[current_line][1])
                    current_line += 1
                setattr(motif, key, str.join("\n", value))
            # Store a matrix
            if action[0] == "store_matrix":
                # First line is alphabet
                alphabet = rest.split()
                alphabet_size = len(alphabet)
                rows = []
                pattern = ""
                current_line += 1
                # Next lines are the rows of the matrix (we allow 0 rows)
                while current_line < len(lines):
                    prefix, rest = lines[current_line]
                    # Prefix should be a two digit 0 padded row number
                    if not prefix.isdigit():
                        break
                    # The first `alphabet_size` fields are the row values
                    values = rest.split()
                    rows.append([float(_) for _ in values[:alphabet_size]])
                    # TRANSFAC includes an extra column with the IUPAC code
                    if len(values) > alphabet_size:
                        pattern += values[alphabet_size]
                    current_line += 1
                # Only store the pattern if it is the correct length (meaning
                # that every row had an extra field)
                if len(pattern) != len(rows):
                    pattern = None
                matrix = FrequencyMatrix.from_rows(alphabet, rows)
                setattr(motif, action[1], matrix)
        # Only return a motif if we saw at least ID or AC or NA
        if motif.id or motif.accession or motif.name:
            return motif


class TransfacWriter(object):
    """
    Writes motifs in TRANSFAC format.
    """

    actions = transfac_actions

    def __init__(self, output):
        self.output = output

    def write(self, motif):
        output = self.output
        for prefix, actions in self.actions.items():
            action = actions[0]
            if action == "store_single":
                key = actions[1]
                if getattr(motif, key) is not None:
                    print(prefix, "  ", getattr(motif, key), file=output)
                    print("XX", file=output)
            elif action == "store_single_list":
                key = actions[1]
                if getattr(motif, key) is not None:
                    value = getattr(motif, key)
                    for v in value:
                        print(prefix, "  ", v, file=output)
                    print("XX", file=output)
            elif action == "store_single_key_value":
                key = actions[1]
                if getattr(motif, key) is not None:
                    value = getattr(motif, key)
                    for k, v in value.items():
                        print(prefix, "  ", "%s=%s" % (k, v), file=output)
                    print("XX", file=output)
            elif action == "store_block":
                key = actions[1]
                if getattr(motif, key) is not None:
                    value = getattr(motif, key)
                    for line in value.split("\n"):
                        print(prefix, "  ", line, file=output)
                    print("XX", file=output)
            elif action == "store_matrix":
                key = actions[1]
                if getattr(motif, key) is not None:
                    matrix = getattr(motif, key)
                    print(prefix, "  ", " ".join(s.rjust(6) for s in matrix.alphabet), file=output)
                    for i in range(matrix.width):
                        print("%02d" % (i + 1), "  ", " ".join(str(matrix.values[i, matrix.char_to_index[ord(s)]]).rjust(6) for s in matrix.alphabet), file=output)
                    print("XX", file=output)
        print("//")



#!/usr/bin/env python

"""
Application to convert AXT file to LAV file. Reads an AXT file from standard
input and writes a LAV file to standard out; some statistics are written to
standard error.

usage: %prog primary_spec secondary_spec [--silent] < axt_file > lav_file

Each spec is of the form seq_file[:species_name]:lengths_file.

- seq_file should be a format string for the file names for the individual
  sequences, with %s to be replaced by the alignment's src field.  For
  example, "hg18/%s.nib" would prescribe files named "hg18/chr1.nib",
  "hg18/chr2.nib", etc.

- species_name is optional.  If present, it is prepended to the alignment's
  src field.

- Lengths files provide the length of each chromosome (lav format needs this
  information but axt file does not contain it).  The format is a series of
  lines of the form:

  <chromosome name> <length>

  The chromosome field in each axt block must match some <chromosome name> in
  the lengths file.
"""

__author__ = "Bob Harris (rsharris@bx.psu.edu)"

import sys

import bx.align.axt
import bx.align.lav


def usage(s=None):
    message = __doc__
    if s is None:
        sys.exit(message)
    else:
        sys.exit("%s\n%s" % (s, message))


def main():
    global debug

    primary = None
    secondary = None
    silent = False

    # pick off options

    args = sys.argv[1:]
    while len(args) > 0:
        arg = args.pop(0)
        val = None
        fields = arg.split("=", 1)
        if len(fields) == 2:
            arg = fields[0]
            val = fields[1]
            if val == "":
                usage("missing a value in %s=" % arg)

        if arg == "--silent" and val is None:
            silent = True
        elif primary is None and val is None:
            primary = arg
        elif secondary is None and val is None:
            secondary = arg
        else:
            usage("unknown argument: %s" % arg)

    if primary is None:
        usage("missing primary file name and length")

    if secondary is None:
        usage("missing secondary file name and length")

    try:
        (primaryFile, primary, primaryLengths) = parse_spec(primary)
    except Exception:
        usage("bad primary spec (must be seq_file[:species_name]:lengths_file")

    try:
        (secondaryFile, secondary, secondaryLengths) = parse_spec(secondary)
    except Exception:
        usage("bad secondary spec (must be seq_file[:species_name]:lengths_file")

    # read the lengths

    speciesToLengths = {}
    speciesToLengths[primary] = read_lengths(primaryLengths)
    speciesToLengths[secondary] = read_lengths(secondaryLengths)

    # read the alignments

    out = bx.align.lav.Writer(
        sys.stdout,
        attributes={
            "name_format_1": primaryFile,
            "name_format_2": secondaryFile})

    axtsRead = 0
    axtsWritten = 0
    for axtBlock in bx.align.axt.Reader(
            sys.stdin,
            species_to_lengths=speciesToLengths,
            species1=primary,
            species2=secondary,
            support_ids=True):
        axtsRead += 1
        out.write(axtBlock)
        axtsWritten += 1

    out.close()

    if not silent:
        sys.stderr.write("%d blocks read, %d written\n" % (axtsRead, axtsWritten))


def parse_spec(spec):  # returns (seq_file,species_name,lengths_file)
    fields = spec.split(":")
    if len(fields) == 2:
        return (fields[0], "", fields[1])
    elif len(fields) == 3:
        return (fields[0], fields[1], fields[2])
    else:
        raise ValueError


def read_lengths(fileName):
    chromToLength = {}

    f = open(fileName, "r")

    for lineNumber, line in enumerate(f):
        line = line.strip()
        if line == "":
            continue
        if line.startswith("#"):
            continue

        fields = line.split()
        if len(fields) != 2:
            raise ValueError("bad lengths line (%s:%d): %s" % (fileName, lineNumber, line))

        chrom = fields[0]
        try:
            length = int(fields[1])
        except ValueError:
            raise ValueError("bad lengths line (%s:%d): %s" % (fileName, lineNumber, line))

        if chrom in chromToLength:
            raise ValueError("%s appears more than once (%s:%d): %s" % (chrom, fileName, lineNumber, line))

        chromToLength[chrom] = length

    f.close()

    return chromToLength


if __name__ == "__main__":
    main()



"""
Classes for generating random sets of intervals over larger regions.
"""

import bisect

from bx.bitset import BitSet

random = __import__('random')


class MaxtriesException(Exception):
    pass


def throw_random_list(lengths, mask, allow_overlap=False):
    rval = []
    throw_random_gap_list(lengths, mask, lambda s, e: rval.append((s, e)), allow_overlap)
    assert sum(b - a for a, b in rval) == sum(lengths)
    return rval


def throw_random_bits(lengths, mask, allow_overlap=False):
    rval = BitSet(mask.size)
    throw_random_gap_list(lengths, mask, lambda s, e: rval.set_range(s, e - s), allow_overlap)
    if not allow_overlap:
        assert rval.count_range(0, rval.size) == sum(lengths)
    return rval


def throw_random_gap_list(lengths, mask, save_interval_func, allow_overlap=False):
    """
    Generates a set of non-overlapping random intervals from a length
    distribution.

    `lengths`: list containing the length of each interval to be generated.
               We expect this to be sorted by decreasing length to minimize
               the chance of failure (MaxtriesException) and for some
               performance gains when allow_overlap==True and there are
               duplicate lengths
    `mask`: a BitSet in which set bits represent regions not to place
            intervals. The size of the region is also determined from the
            mask.
    """
    # Use mask to find the gaps;  gaps is a list of (length,start,end)
    lengths = [length for length in lengths if length > 0]
    min_length = min(lengths)
    gaps = []
    start = end = 0
    while True:
        start = mask.next_clear(end)
        if start == mask.size:
            break
        end = mask.next_set(start)
        if end-start >= min_length:
            gaps.append((end-start, start, None))
    # Sort (long regions first)
    gaps.sort()
    gaps.reverse()
    # Throw
    throw_random_private(lengths, gaps, save_interval_func, allow_overlap, three_args=False)


def throw_random_intervals(lengths, regions, save_interval_func=None, allow_overlap=False):
    """
    Generates a set of non-overlapping random intervals from a length
    distribution.

    `lengths`: list containing the length of each interval to be generated.
               We expect this to be sorted by decreasing length to minimize
               the chance of failure (MaxtriesException) and for some
               performance gains when allow_overlap==True and there are
               duplicate lengths.
    `regions`: A list of regions in which intervals can be placed.  Elements
               are tuples or lists of the form (start, end, ...), where ...
               indicates any number of items (including zero).
    `save_interval_func`: A function accepting three arguments which will be
                          passed the (start,stop,region) for each generated
                          interval, where region is an entry in the regions
                          list.  If this is None, the generated intervals will
                          be returned as a list of elements copied from the
                          region with start and end modified.
    """
    # Copy regions
    regions = sorted([(x[1]-x[0], x[0], x) for x in regions])
    # Sort (long regions first)
    regions.reverse()
    # Throw
    if (save_interval_func is not None):
        throw_random_private(lengths, regions, save_interval_func, allow_overlap)
        return
    else:
        intervals = []

        def save_interval_func(s, e, rgn):
            return intervals.append(overwrite_start_end(s, e, rgn))

        throw_random_private(lengths, regions, save_interval_func, allow_overlap)
        return intervals


def overwrite_start_end(s, e, rgn):
    rgn = list(rgn)
    rgn[0] = s
    rgn[1] = e
    return tuple(rgn)


def throw_random_private(lengths, regions, save_interval_func, allow_overlap=False, three_args=True):
    """
    (Internal function;  we expect calls only through the interface functions
    above)

    `lengths`: A list containing the length of each interval to be generated.
    `regions`: A list of regions in which intervals can be placed, sorted by
               decreasing length.  Elements are triples of the form (length,
               start, extra), This list CAN BE MODIFIED by this function.
    `save_interval_func`: A function accepting three arguments which will be
                          passed the (start,stop,extra) for each generated
                          interval.
    """

    # Implementation:
    #   We keep a list of the regions, sorted from largest to smallest.  We then
    #   place each length by following steps:
    #     (1) construct a candidate counts array (cc array)
    #     (2) choose a candidate at random
    #     (3) find region containing that candidate
    #     (4) map candidate to position in that region
    #     (5) split region if not allowing overlaps
    #     (6) report placed segment
    #
    #   The cc array is only constructed if there's a change (different length
    #   to place, or the region list has changed).  It contains, for each
    #   region, the total number of number of candidate positions in regions
    #   *preceding* it in the region list:
    #     cc[i] = sum over k in 0..(i-1) of length[i] - L + 1
    #   where N is the number of regions and L is the length being thrown.
    #   At the same time, we determine the total number of candidates (the total
    #   number of places the current length can be placed) and the index range
    #   of regions into which the length will fit.
    #
    #   example:
    #     for L = 20
    #     i =           0   1   2   3   4   5   6   7   8   9
    #     length[i] =  96  66  56  50  48  40  29  17  11   8
    #     cc[i] =       0  77 124 161 192 221 242   X   X   X
    #     candidates = 252
    #     lo_rgn = 0
    #     hi_rgn = 6
    #
    #   The candidate is chosen in (0..candidates-1).  The candidate counts
    #   array allows us to do a binary search to locate the region that holds that
    #   candidate.  Continuing the example above, we choose a random candidate
    #   s in (0..251).  If s happens to be in (124..160), it will be mapped to
    #   region 2 at start position s-124.
    #
    #   During the binary search, if we are looking at region 3, if s < cc[3]
    #   then the desired region is region 2 or lower.  Otherwise it is region 3 or
    #   higher.

    min_length = min(lengths)
    prev_length = None  # (force initial cc array construction)
    cc = [0] * (len(regions) + len(lengths) - 1)
    num_thrown = 0
    for length in lengths:
        # construct cc array (only needed if length has changed or region list has
        # changed)
        if length != prev_length:
            prev_length = length
            assert len(cc) >= len(regions)
            candidates = 0
            hi_rgn = 0
            for region in regions:
                rgn_len = region[0]
                if rgn_len < length:
                    break
                cc[hi_rgn] = candidates
                candidates += rgn_len - length + 1
                hi_rgn += 1
            if candidates == 0:
                raise MaxtriesException(
                    "No region can fit an interval of length %d (we threw %d of %d)"
                    % (length, num_thrown, len(lengths)))
            hi_rgn -= 1
        # Select a candidate
        s = random.randrange(candidates)
        # ..
        # ..for ix in range( len( regions ) ):
        # ..    region = regions[ix]
        # ..    if ix <= hi_rgn: print "%2s: %5s %5s %5s" % ( ix, region[1], region[0], cc[ix] )
        # ..    else:            print "%2s: %5s %5s %5s" % ( ix, region[1], region[0], "X" )
        # ..print "s = %s (of %s candidates)" % ( s, candidates )
        # Locate region containing that candidate, by binary search
        lo = 0
        hi = hi_rgn
        while hi > lo:
            mid = (lo + hi + 1) / 2     # (we round up to prevent infinite loop)
            if s < cc[mid]:
                hi = mid-1  # (s <  num candidates from 0..mid-1)
            else:
                lo = mid    # (s >= num candidates from 0..mid-1)
        s -= cc[lo]
        # If we are not allowing overlaps we will remove the placed interval
        # from the region list
        if allow_overlap:
            rgn_length, rgn_start, rgn_extra = regions[lo]
        else:
            # Remove the chosen region and split
            rgn_length, rgn_start, rgn_extra = regions.pop(lo)
            rgn_end = rgn_start + rgn_length
            assert s >= 0
            assert rgn_start + s + length <= rgn_end, "Expected: %d + %d + %d == %d <= %d" % (rgn_start, s, length, rgn_start + s + length, rgn_end)
            regions.reverse()
            if s >= min_length:
                bisect.insort(regions, (s, rgn_start, rgn_extra))
            if s + length <= rgn_length - min_length:
                bisect.insort(regions, (rgn_length - (s + length), rgn_start + s + length, rgn_extra))
            regions.reverse()
            prev_length = None  # (force cc array construction)
        # Save the new interval
        if (three_args):
            save_interval_func(rgn_start + s, rgn_start + s + length, rgn_extra)
        else:
            save_interval_func(rgn_start + s, rgn_start + s + length)
        num_thrown += 1



#!/usr/bin/env python2.3

"""
Read a MAF from stdin and break into several mafs based on the source of
each block. If the `component` option is provided then only that component
will be used to determine the new file for each block, otherwise the src
for *all* components will be used.

TODO: Should be able to specify component by species/prefix?

usage: %prog [options] < maf
    -o, --outprefix: prepend this to the name of each generate maf
    -c, --component: use only this component (by index!) to split
"""

import string
import sys
from optparse import OptionParser

import bx.align.maf


INF = "inf"


def __main__():

    # Parse command line arguments

    parser = OptionParser()
    parser.add_option("-o", "--outprefix", action="store", default="")
    parser.add_option("-c", "--component", action="store", default=None)
    (options, args) = parser.parse_args()

    out_prefix = options.outprefix
    comp = options.component
    if comp is not None:
        comp = int(comp)

    maf_reader = bx.align.maf.Reader(sys.stdin)

    writers = {}

    for m in maf_reader:

        if comp is None:
            writer_key = string.join([c.src for c in m.components], '_')
        else:
            writer_key = m.components[comp].src

        if writer_key not in writers:
            writer = bx.align.maf.Writer(open("%s%s.maf" % (out_prefix, writer_key), "w"))
            writers[writer_key] = writer
        else:
            writer = writers[writer_key]

        writer.write(m)

    for key in writers:
        writers[key].close()


if __name__ == "__main__":
    __main__()



#!/usr/bin/env python

"""
Script to select random lines from a file. Reads entire file into
memory!

TODO: Replace this with a more elegant implementation.
"""
from __future__ import print_function

import random
import sys

ndesired = int(sys.argv[1])

for line in random.sample(sys.stdin.readlines(), ndesired):
    print(line, end=' ')



#!/usr/bin/env python2.3

"""
Read a PAIRWISE maf from stdin and print the percent identity of each
alignment, where percent identity is defined as the number of matching columns
over the number of aligned (non-gap) columns.

TODO: Generalize for more than two species

usage: %prog < maf > out
"""
from __future__ import division, print_function

import sys

from bx.align import maf


def __main__():

    maf_reader = maf.Reader(sys.stdin)

    for m in maf_reader:
        match = 0
        total = 0
        for i in range(0, m.text_size):
            a = m.components[0].text[i].lower()
            b = m.components[1].text[i].lower()
            if a == '-' or b == '-':
                continue
            elif a == b:
                match += 1
            total += 1

        print(match / total)


if __name__ == "__main__":
    __main__()



#!/usr/bin/env python

"""
Read a maf file from stdin and write out a new maf with only blocks having all of
the passed in species, after dropping any other species and removing columns
containing only gaps. By default this will attempt to fuse together any blocks
which are adjacent after the unwanted species have been dropped.

usage: %prog species1 species2 ... < maf
    -n, --nofuse: Don't attempt to join blocks, just remove rows.
"""

import sys

import bx.align.maf
from bx.align.tools.fuse import FusingAlignmentWriter
from bx.align.tools.thread import (
    get_components_for_species,
    remove_all_gap_columns
)
from bx.cookbook import doc_optparse


def main():

    options, args = doc_optparse.parse(__doc__)

    try:
        species = args
        # Allow a comma separated list, TODO: allow a newick format tree
        if len(species) == 1 and ',' in species[0]:
            species = species[0].split(',')
        fuse = not(bool(options.nofuse))
    except Exception:
        doc_optparse.exit()

    maf_reader = bx.align.maf.Reader(sys.stdin)
    maf_writer = bx.align.maf.Writer(sys.stdout)

    if fuse:
        maf_writer = FusingAlignmentWriter(maf_writer)

    for m in maf_reader:
        new_components = get_components_for_species(m, species)
        if new_components:
            remove_all_gap_columns(new_components)
            m.components = new_components
            m.score = 0.0
            maf_writer.write(m)

    maf_reader.close()
    maf_writer.close()


if __name__ == "__main__":
    main()



#!/usr/bin/env python2.3

"""
Read a MAF from standard input and determine the mean length of ungapped pieces
in each block.

usage: %prog < maf > out
"""
from __future__ import division, print_function

import sys

import bx.align.maf


def main():

    for m in bx.align.maf.Reader(sys.stdin):

        ungapped_columns = 0
        ungapped_runs = 0
        in_ungapped = False

        for col in m.column_iter():
            is_gap = ('-' in col)
            if not is_gap:
                ungapped_columns += 1
            if in_ungapped and is_gap:
                ungapped_runs += 1
            in_ungapped = not is_gap
        if in_ungapped:
            ungapped_runs += 1

        print(ungapped_columns / ungapped_runs)


if __name__ == "__main__":
    main()



"""
Tools for counting words (n-grams) in integer sequences.
"""

import numpy

cdef extern from "Python.h":
    ctypedef int Py_intptr_t

# cdef extern from "numpy/npy_3kcompat.h":
    # NOTE: including npy_3kcompat.h did not compile,
    #       so use the explicitly extracted function from here:
cdef extern from "npy_capsule_as_void_ptr.h":
    void * NpyCapsule_AsVoidPtr(object) except NULL

# for PyArrayInterface:
CONTIGUOUS=0x01
FORTRAN=0x02
ALIGNED=0x100
NOTSWAPPED=0x200
WRITEABLE=0x400

ctypedef struct PyArrayInterface:
    int two              # contains the integer 2 as a sanity check
    int nd               # number of dimensions
    char typekind        # kind in array --- character code of typestr
    int itemsize         # size of each element
    int flags            # flags indicating how the data should be interpreted
    Py_intptr_t *shape   # A length-nd array of shape information
    Py_intptr_t *strides # A length-nd array of stride information
    void *data           # A pointer to the first element of the array
    
def count_ngrams( object ints, int n, int radix ):
    """
    Count the number of occurrences of each possible length `n` word in 
    `ints` (which contains values from 0 to `radix`). Returns an array
    of length `radix` ** `n` containing the counts.
    """
    cdef PyArrayInterface * ints_desc
    cdef PyArrayInterface * rval_desc
    # Get array interface for input string and validate
    ints_desc_obj = ints.__array_struct__
    ints_desc = <PyArrayInterface *> NpyCapsule_AsVoidPtr( ints_desc_obj )
    assert ints_desc.two == 2, "Array interface sanity check failed, got %d" % ints_desc.two
    assert ints_desc.nd == 1, "Input array must be 1d"
    assert ints_desc.typekind == 'i'[0], "Input array must contain integers"
    assert ints_desc.itemsize == 4, "Input array must contain 32bit integers"
    assert ints_desc.flags & CONTIGUOUS > 0, "Input array must be contiguous"
    assert ints_desc.flags & ALIGNED > 0, "Input array must be aligned"
    assert ints_desc.flags & NOTSWAPPED > 0, "Input array must not be byteswapped"
    # Create numpy array for return value, get array interface and validate
    rval = numpy.zeros( <int> ( ( <float> radix ) ** n ), dtype=numpy.int32 )
    assert ints_desc.two == 2, "Array interface sanity check failed, got %d" % ints_desc.two
    rval_desc_obj = rval.__array_struct__
    rval_desc = <PyArrayInterface *> NpyCapsule_AsVoidPtr( rval_desc_obj )
    assert rval_desc.two == 2, "Array interface sanity check failed"
    assert rval_desc.nd == 1, "Input array must be 1d"
    assert rval_desc.typekind == 'i'[0], "Input array must contain integers"
    assert rval_desc.itemsize == 4, "Input array must contain 32bit integers"
    assert rval_desc.flags & CONTIGUOUS > 0, "Input array must be contiguous"
    assert rval_desc.flags & ALIGNED > 0, "Input array must be aligned"
    assert rval_desc.flags & NOTSWAPPED > 0, "Input array must not be byteswapped"
    # Do it
    _count_ngrams( <int*> ints_desc.data, ints_desc.shape[0], <int*> rval_desc.data, n, radix )
    return rval
    
cdef _count_ngrams( int* ints, int n_ints, int* rval, int n, int radix ):
    cdef int i, j, index, factor, letter
    # Loop over each word in the string
    for i from 0 <= i < ( n_ints - n ):
        # Walk back to build index into count array
        index = 0
        factor = 1
        for j from 0 <= j < n:
            letter = ints[ i + j ]
            if letter < 0 or letter >= radix:
                # This word is bad, break out and do not increment counts
                print "breaking, letter", letter
                break
            index = index + letter * factor
            factor = factor * radix
        else:
            print index
            rval[ index ] = rval[ index ] + 1



#!/usr/bin/env python

"""
Create a site profile vector showing the average signal accumulated from a
bigwig file around the center of each interval from a BED file.

Output is the average signal value at that relative position across the
intervals.

usage: %prog bigwig_file.bw padding < bed_file.bed
"""

import sys

from numpy import (
    float64,
    floor,
    int32,
    isnan,
    savetxt,
    zeros
)

from bx.bbi.bigwig_file import BigWigFile
from bx.intervals.io import GenomicIntervalReader

bw = BigWigFile(open(sys.argv[1]))
padding = int(sys.argv[2])
totals = zeros(padding*2, dtype=float64)
valid = zeros(padding*2, dtype=int32)

for interval in GenomicIntervalReader(sys.stdin):
    center = floor((interval.start + interval.end) / 2)
    values = bw.get_as_array(interval.chrom, center - padding, center + padding)
    # Determine which positions had data and mask the rest for totalling
    invalid = isnan(values)
    values[invalid] = 0
    totals += values
    valid += (~ invalid)

savetxt(sys.stdout, totals/valid)



"""
Classes for char-to-int mapping and int-to-int mapping.

:Author: James Taylor (james@bx.psu.edu)

The char-to-int mapping can be used to translate a list of strings
over some alphabet to a single int array (example for encoding a multiple
sequence alignment).

The int-to-int mapping is particularly useful for creating partitions,
and provides methods to merge/split symbols in the output mapping.

The two forms of mapping can be combined, for example to encode a
multiple sequence alignment in a reduced alphabet defined by a partition
of alignment columns. Many of the helper methods provided are for
solving such alignment oriented problems.

This code was originally written for the `ESPERR`_ project which includes
software for searcing for alignment encodings that work well for specific
classification problems using various Markov chain classifiers over the
reduced encodings.

Most of the core implementation is in the pyrex/C extension
"_seqmapping.pyx" for performance reasons (specifically to avoid the
excessive bounds checking that would make a sequence/array lookup heavy
problem like this slow in pure python).

.. _ESPERR: http://www.bx.psu.edu/projects/esperr/
"""

from ._seqmapping import (
    CharToIntArrayMapping,
    IntToIntMapping,
)

# Char->Int mapping for DNA characters with missing data

DNA = CharToIntArrayMapping()
DNA.set_mapping("a", 0)
DNA.set_mapping("A", 0)
DNA.set_mapping("c", 1)
DNA.set_mapping("C", 1)
DNA.set_mapping("g", 2)
DNA.set_mapping("G", 2)
DNA.set_mapping("t", 3)
DNA.set_mapping("T", 3)
DNA.set_mapping("-", 4)
DNA.set_mapping("*", 5)

# Creating mappings


def alignment_mapping_from_file(f, char_mapping=DNA):
    """
    Create a mapping from a file of alignment columns.
    """
    columns, symbols = [], []
    for line in f:
        column, symbol = line.split()
        columns.append(column)
        symbols.append(int(symbol))

    align_count = len(columns[0])

    mapping = IntToIntMapping(char_mapping.get_out_size() ** align_count)

    for column, symbol in zip(columns, symbols):
        index = char_mapping.translate_list(list(column))[0]
        mapping.set_mapping(index, symbol)

    return align_count, mapping


def second_mapping_from_file(f, first_mapping, char_mapping=DNA):

    columns, symbols = [], []
    for line in f:
        column, symbol = line.split()
        columns.append(column)
        symbols.append(int(symbol))

    mapping = IntToIntMapping(first_mapping.get_out_size())

    for column, symbol in zip(columns, symbols):
        index = char_mapping.translate_list(list(column))[0]
        if first_mapping[index] >= 0:
            mapping.set_mapping(first_mapping[index], symbol)

    return mapping


def identity_mapping(size):
    mapping = IntToIntMapping(size)
    for i in range(size):
        mapping.set_mapping(i, i)
    return mapping



"""
Tests for `bx.align.maf.sitemask`.
"""
import tempfile

from six import StringIO

import bx.align.maf
from . import cpg

test_maf_cpg = """##maf version=1 scoring=none
a score=0
s apple  34 64 + 110 AGGGA---GTTCGTCACT------GTCGTAAGGGTTCAGA--CTGTCTATGTATACACAAGTTGTGTTGCA--ACCG
s orange 19 61 - 100 AGGGATGCGTT--TCACTGCTATCGTCGTA----TTCAGACTTCG-CTATCT------GAGTTGT---GCATTACCG
"""

cpg_inclusive_result = [
    "##maf,version=1",
    "a,score=0",
    "s,apple,34,64,+,110,AGGGA---GTTCGTCACT------GT##TAAGGGTTCAGA--CTGTCTATGTATACACAAGTTGTGTTGCA--ACCG",
    "s,orange,19,61,-,100,AGGGATG#GTT--TCACTGCTAT#GT##TA----TTCAGACTTCG-CTATCT------GAGTTGT---GCATTACCG"
]

cpg_restricted_result = [
    "##maf,version=1",
    "a,score=0",
    "s,apple,34,64,+,110,A##GA---#TT##TC#C#------#T##TA###GTTC#GA--C##TC#A#G#ATAC####GT#G#GT#GC#--AC#G",
    "s,orange,19,61,-,100,A##GA#G##TT--TC#C#GC#AT##T##TA----TTC#GAC#T##-C#A#C#------##GT#G#---GC#TTAC#G"
]

noncpg_result = [
    "##maf,version=1",
    "a,score=0",
    "s,apple,34,64,+,110,#GG##---G##CG##A#T------G#CG##AGG####A##--#TG##T#T#T####ACAA##T#T##T##A--##CG",
    "s,orange,19,61,-,100,#GG##T#CG##--##A#T##T##CG#CG##----###A###T#CG-#T#T#T------GA##T#T---##A####CG"
]


def test_cpg_inclusive():
    reader = bx.align.maf.Reader(StringIO(test_maf_cpg))
    out = tempfile.NamedTemporaryFile('w')
    writer = bx.align.maf.Writer(out)
    cpgfilter = cpg.Inclusive(mask='#')
    cpgfilter.run(reader, writer.write)
    out.seek(0)
    j = 0
    for line in open(out.name):
        line = line.strip()
        if not(line):
            continue
        assert cpg_inclusive_result[j] == ",".join(line.split())
        j += 1


def test_cpg_restricted():
    reader = bx.align.maf.Reader(StringIO(test_maf_cpg))
    out = tempfile.NamedTemporaryFile('w')
    writer = bx.align.maf.Writer(out)
    cpgfilter = cpg.Restricted(mask='#')
    cpgfilter.run(reader, writer.write)
    out.seek(0)
    j = 0
    for line in open(out.name):
        line = line.strip()
        if not(line):
            continue
        assert cpg_restricted_result[j] == ",".join(line.split())
        j += 1


def test_non_cpg():
    reader = bx.align.maf.Reader(StringIO(test_maf_cpg))
    out = tempfile.NamedTemporaryFile('w')
    writer = bx.align.maf.Writer(out)
    cpgfilter = cpg.nonCpG(mask='#')
    cpgfilter.run(reader, writer.write)
    out.seek(0)
    j = 0
    for line in open(out.name):
        line = line.strip()
        if not(line):
            continue
        assert noncpg_result[j] == ",".join(line.split())
        j += 1



#!/usr/bin/env python2.4

"""
Read a compressed file as created by 'lzop' from stdin and write a table to
stdout containing the blocksize and the start offset (in bytes) of each
compressed block.

usage: %prog < FILENAME.lzo > FILENAME.lzot
"""
from __future__ import print_function

import struct
import sys

MAGIC = "\x89\x4c\x5a\x4f\x00\x0d\x0a\x1a\x0a"

F_ADLER32_D = 0x00000001
F_ADLER32_C = 0x00000002
F_H_EXTRA_FIELD = 0x00000040
F_H_GMTDIFF = 0x00000080
F_CRC32_D = 0x00000100
F_CRC32_C = 0x00000200
F_MULTIPART = 0x00000400
F_H_FILTER = 0x00000800
F_H_CRC32 = 0x00001000

assert struct.calcsize("!H") == 2
assert struct.calcsize("!I") == 4


class UnpackWrapper(object):
    def __init__(self, file):
        self.file = file

    def read(self, amt):
        return self.file.read(amt)

    def get(self, fmt):
        t = struct.unpack(fmt, self.file.read(struct.calcsize(fmt)))
        return t[0]


def main():
    f = UnpackWrapper(sys.stdin)
    # Read header
    magic = f.read(9)
    assert magic == MAGIC, "Not LZOP file"
    version = f.get("!H")
    f.get("!H")  # lib_version
    if version >= 0x0940:
        f.get("!H")  # extract_version
    method = f.get("!B")
    assert 1 <= method <= 3, "Only LZO compression is currently supported"
    f.get("!B")  # level
    flags = f.get("!I")
    assert not(flags & F_H_FILTER), "LZOP filters not supported"
    has_compressed_crc = (flags & F_CRC32_C or flags & F_ADLER32_C)
    has_uncompressed_crc = (flags & F_CRC32_D or flags & F_ADLER32_D)
    f.get("!I")  # mode
    f.get("!I")  # time
    f.get("!I")  # time_offset
    fname_len = f.get("!B")
    fname = f.read(fname_len)
    assert len(fname) == fname_len, "EOF reading filename"
    f.get("!I")  # header_crc
    if flags & F_H_EXTRA_FIELD:
        extra_len = f.get("!I")
        extra = f.read(extra_len)
        assert len(extra) == extra_len, "EOF reading extra field"
    # Done with header
    block_size = None
    expect_no_more = False
    # Read blocks
    while True:
        size = f.get("!I")
        if size == 0:
            break
        assert not(expect_no_more), \
            "Encountered an undersized block that was not the last block"
        if block_size is None:
            print("s", size)
            block_size = size
        else:
            if size < block_size:
                expect_no_more = True
        compressed_size = f.get("!I")
        if has_uncompressed_crc:
            f.get("!I")  # crc
        if has_compressed_crc:
            f.get("!I")  # compressed_crc
        print("o", f.file.tell(), compressed_size, size)
        compressed_data = f.read(compressed_size)
        assert len(compressed_data) == compressed_size, \
            "EOF reading compressed data"


if __name__ == "__main__":
    main()





import logging, gzip
from collections import namedtuple
import numpy
cimport numpy

log = logging.getLogger(__name__)

cimport cython

DTYPE = numpy.uint64


cdef inline int max2( int a, int b ):
    if b > a:
        return b
    return a

cdef inline int min2( int a, int b ):
    if b < a:
        return b
    return a


def rem_dash(p, q):
    """remove dash columns and shift match intervals to the left. both iterables
    are read on the same direction left-to-right.
    """

    def myp(l):
        if l: return l.pop(0)

    def adv(queue, i, d):
        # shifted interval
        shi = i[0]-d, i[1]-d
        assert shi[0] >= 0
        if queue and queue[-1][1] == shi[0]:
            # join to the preceeding one
            queue[-1] = (queue[-1][0], shi[1])
        else:
            queue.append( shi )
        return queue

    p_card = sum( map(lambda i: p[i][1] - p[i][0], range(len(p))) )
    q_card = sum( map(lambda i: q[i][1] - q[i][0], range(len(q))) )

    P, Q = [], []
    dash = 0 # dash (on both cigars) count so far
    a, b = p.pop(0), q.pop(0)
    #while p or q:
    while a and b:
        assert dash <= min(a[0], b[0])
        i = max(a[0], b[0]) - min(a[1], b[1])
        if i >= 0: # no intersection
            if a[1] <= b[0]:
                if p:
                    i = min(i, p[0][0] - a[1])
                P = adv(P, a, dash)
                a = myp(p)
            else:
                if q:
                    i = min(i, q[0][0] - b[1])
                Q = adv(Q, b, dash)
                b = myp(q)
            dash += i
        else: # intersection
            if a[1] >= b[1]:
                Q = adv(Q, b, dash); b = myp(q)
            elif a[1] < b[1]:
                P = adv(P, a, dash); a = myp(p)
        #if not a or not b: # no more matchings
        #    break
    assert (not p) or (not q), "one or both should be empty: p=%s, q=%s" % (str(p), str(q))

    if a: P = adv(P, a, dash)
    if b: Q = adv(Q, b, dash)

    # remaining intervals (in q or p)
    r, R = p, P
    if q: r, R = q, Q
    # just extend the last inteval by the remaining bases
    R[-1] = (R[-1][0], R[-1][1] + sum( map(lambda i: i[1]-i[0], r) ))

    P_card = sum( map(lambda i: P[i][1] - P[i][0], range(len(P))) )
    Q_card = sum( map(lambda i: Q[i][1] - Q[i][0], range(len(Q))) )

    assert p_card == P_card, "%d != %d" % (p_card, P_card)
    assert q_card == Q_card, "%d != %d" % (q_card, Q_card)

    return P, Q

def fastLoadChain(fname, hf):
    data = []
    open_f = (fname.endswith(".gz") and gzip.open or open)
    with open_f(fname, "rt") as fd:
        while True:
            line = fd.readline()
            if line == "":
                break
            hd = hf(line)
            N = []
            line = fd.readline().split()
            while len(line) == 3:
                N.append( (int(line[0]), int(line[1]), int(line[2])) )
                line = fd.readline().split()
            if len(line) != 1:
                raise ValueError("last matching block expected (found %s)" % str(line))
            N.append( (int(line[0]), 0, 0) )
            s, t, q = zip( *N )
            data.append( (hd,
                numpy.array(s, dtype=numpy.int),
                numpy.array(t, dtype=numpy.int),
                numpy.array(q, dtype=numpy.int)) )
            assert hd.tEnd - hd.tStart == sum(s) + sum(t)
            assert hd.qEnd - hd.qStart == sum(s) + sum(q)
            fd.readline() # a blank line
        log.info("parsed %d elements from %s" % (len(data), fname))
    return data



@cython.wraparound(False)
@cython.boundscheck(False)
cpdef numpy.ndarray[numpy.uint64_t, ndim=2] bed_union( numpy.ndarray[numpy.uint64_t, ndim=2] elements ):
    """compute the union of these elements. simply walk the sorted elements and join the intersecting ones
    works on half-open intervals, i.e., [a, b), [b, c) ---> [a, c)

    @param elements: 2-dim numpy array of unsigned64 ints
    @return: 2-dim numpy array of unsigned64 ints"""

    assert numpy.shape(elements)[0] > 0

    cdef Py_ssize_t cst, cen, i, j
    cdef numpy.ndarray[numpy.uint64_t, ndim=2] tmp_elems, final_elems

    elements.sort(axis=0)
    assert elements[0][0] <= elements[numpy.shape(elements)[0]-1][0]
    tmp_elems = numpy.zeros((numpy.shape(elements)[0], 2), dtype=DTYPE)
    cst = elements[0, 0]
    cen = elements[0, 1]
    j = 0

    for i in range(1, numpy.shape(elements)[0]):
        if elements[i, 0] <= cen: # overlaps with the last one
            cen = max2(cen, elements[i, 1])
        else:
            tmp_elems[j, 0] = cst
            tmp_elems[j, 1] = cen
            j += 1
            cst = elements[i, 0]
            cen = elements[i, 1]
    tmp_elems[j, 0] = cst
    tmp_elems[j, 1] = cen
    j += 1
    final_elems = numpy.empty((j, 2), dtype=DTYPE)
    for i in range(j):
        final_elems[i, 0] = tmp_elems[i, 0]
        final_elems[i, 1] = tmp_elems[i, 1]
    assert final_elems[0, 0] == elements[0, 0], "fe=%d, e=%d" % (final_elems[0,0], elements[0,0])
    return final_elems

#@cython.wraparound(False)
#@cython.boundscheck(False)
cpdef numpy.ndarray[numpy.int64_t, ndim=2] cummulative_intervals(numpy.ndarray[numpy.int64_t, ndim=1] S,
        numpy.ndarray[numpy.int64_t, ndim=1] D ):
    """compute cummulative intervals for this side of an aligmnent. S and D are one side of
    the alignment as described in the chain file format"""

    cdef int N = S.shape[0]
    cdef int i = 0, j = 0
    assert N  == D.shape[0]
    cdef numpy.ndarray[numpy.int64_t, ndim=2] cumm_i = numpy.empty((N, 2), dtype=numpy.int64)

    cumm_i[0,0] = 0
    cumm_i[0,1] = S[0]
    for i in range(N-1):
        j = i + 1
        cumm_i[j,0] = cumm_i[i, 1] + D[i]
        cumm_i[j,1] = cumm_i[j,0] + S[j]
    return cumm_i






"""
Determine the number of bases covered by a set of intervals.
"""

from bx.intervals.io import BitsetSafeReaderWrapper
from bx.intervals.operations import MAX_END


def base_coverage(reader):
    # Handle any ValueError, IndexError and OverflowError exceptions that may be thrown when
    # the bitsets are being created by skipping the problem lines
    base_reader = BitsetSafeReaderWrapper(reader, lens={})
    bitsets = base_reader.binned_bitsets()
    coverage = 0
    for chrom in bitsets:
        try:
            coverage += bitsets[chrom].count_range(0, MAX_END)
        except IndexError as e:
            base_reader.skipped += 1
            # no reason to stuff an entire bad file into memmory
            if base_reader.skipped < 10:
                base_reader.skipped_lines.append((base_reader.linenum, base_reader.current_line, str(e)))
            continue
    return coverage



#!/usr/bin/env python2.4
from __future__ import print_function

from optparse import OptionParser

from rpy import r

import bx.align.maf
import bx.bitset
from bx.bitset_builders import binned_bitsets_from_file


def main():
    parser = OptionParser(usage="usage: %prog [options] maf_file snp_file neutral_file window_size step_size")
    parser.add_option("-o", "--outfile", help="Specify file for output")
    parser.add_option("-s", "--species", type="string", default="panTro2")
    parser.add_option("-b", "--build", type="string", default="hg18")
    (options, args) = parser.parse_args()

    if len(args) != 5:
        parser.error("Incorrect number of arguments")
    else:
        maf_filename = args[0]
        snp_filename = args[1]
        neutral_filename = args[2]
        window_size = int(args[3])
        step_size = int(args[4])

    if options.outfile is not None:
        out_file = open(options.outfile, 'w')

    # Generate snp and neutral bitsets
    AR_snp_bitsets = binned_bitsets_from_file(open(snp_filename))
    neutral_bitsets = binned_bitsets_from_file(open(neutral_filename))

    # Generate divergence bitset from maf file
    AR_div_bitsets = dict()
    chr_lens = dict()
    reader = bx.align.maf.Reader(open(maf_filename))

    for block in reader:
        comp1 = block.get_component_by_src_start(options.build)
        comp2 = block.get_component_by_src_start(options.species)

        if comp1 is None or comp2 is None:
            continue

        # Chromosome, start, and stop of reference species alignment
        chr = comp1.src.split('.')[1]
        start = comp1.start

        # Get or create bitset for this chromosome
        if chr in AR_div_bitsets:
            bitset = AR_div_bitsets[chr]
        else:
            bitset = AR_div_bitsets[chr] = bx.bitset.BinnedBitSet()
            chr_lens[chr] = comp1.get_src_size()

        # Iterate over text and set diverged bit
        pos = start
        for ch1, ch2 in zip(comp1.text.upper(), comp2.text.upper()):
            if ch1 == '-':
                continue
            if ch2 == '-':
                pos += 1
                continue

            if ch1 != ch2 and not AR_snp_bitsets[chr][pos]:
                bitset.set(pos)
            pos += 1

    # Debugging Code
#     for chr in AR_div_bitsets:
#         for pos in range(0, AR_div_bitsets[chr].size):
#             if AR_div_bitsets[pos]:
#                 print >> sys.stderr, chr, pos, pos+1

    # Copy div and snp bitsets
    nonAR_snp_bitsets = dict()
    for chr in AR_snp_bitsets:
        nonAR_snp_bitsets[chr] = bx.bitset.BinnedBitSet()
        nonAR_snp_bitsets[chr].ior(AR_snp_bitsets[chr])

    nonAR_div_bitsets = dict()
    for chr in AR_div_bitsets:
        nonAR_div_bitsets[chr] = bx.bitset.BinnedBitSet()
        nonAR_div_bitsets[chr].ior(AR_div_bitsets[chr])

    # Generates AR snps by intersecting with neutral intervals
    for chr in AR_snp_bitsets:
        AR_snp_bitsets[chr].iand(neutral_bitsets[chr])

    # Generates AR divs by intersecting with neutral intervals
    for chr in AR_div_bitsets:
        AR_div_bitsets[chr].iand(neutral_bitsets[chr])

    # Inverts the neutral intervals so now represents nonAR
    for chr in neutral_bitsets:
        neutral_bitsets[chr].invert()

    # Generates nonAR snps by intersecting with masked neutral intervals
    for chr in nonAR_snp_bitsets:
        nonAR_snp_bitsets[chr].iand(neutral_bitsets[chr])
    # Generates nonAR divs by intersecting with masked neutral intervals
    for chr in nonAR_div_bitsets:
        nonAR_div_bitsets[chr].iand(neutral_bitsets[chr])

    for chr in AR_div_bitsets:
        for window in range(0, chr_lens[chr] - window_size, step_size):
            # neutral_size = neutral_bitsets[chr].count_range(window, window_size)
            # if neutral_size < 9200: continue
            AR_snp = AR_snp_bitsets[chr].count_range(window, window_size)
            AR_div = AR_div_bitsets[chr].count_range(window, window_size)
            nonAR_snp = nonAR_snp_bitsets[chr].count_range(window, window_size)
            nonAR_div = nonAR_div_bitsets[chr].count_range(window, window_size)

            if nonAR_snp >= 6 and nonAR_div >= 6 and AR_snp >= 6 and AR_div >= 6:
                MK_pval = MK_chi_pvalue(nonAR_snp, nonAR_div, AR_snp, AR_div)
            else:
                MK_pval = MK_fisher_pvalue(nonAR_snp, nonAR_div, AR_snp, AR_div)

            if options.outfile is not None:
                out_file.write("%s\t%d\t%d\t%d\t%d\t%d\t%d\t%1.15f\n" % (chr, window, window+window_size, nonAR_snp, nonAR_div, AR_snp, AR_div, MK_pval))
            else:
                print("%s\t%d\t%d\t%d\t%d\t%d\t%d\t%1.15f" % (chr, window, window+window_size, nonAR_snp, nonAR_div, AR_snp, AR_div, MK_pval))

    if options.outfile is not None:
        out_file.close()


def MK_fisher_pvalue(win_snp, win_div, AR_snp, AR_div):

    if win_snp == 0 and win_div == 0 and AR_snp == 0 and AR_div == 0:
        return 1.0

    fisher_result = r.fisher_test(r.matrix(r.c([win_snp, win_div, AR_snp, AR_div]), nr=2))

    return fisher_result['p.value']


def MK_chi_pvalue(win_snp, win_div, AR_snp, AR_div):

    chi_result = r.chisq_test(r.matrix(r.c([win_snp, win_div, AR_snp, AR_div]), nr=2))

    return chi_result['p.value']


main()



"""
Classes to support "biological sequence" files.

:Author: Bob Harris (rsharris@bx.psu.edu)
"""
from six import Iterator

# DNA reverse complement table

DNA_COMP = "                                             -                  " \
           " TVGH  CD  M KN   YSA BWXR       tvgh  cd  m kn   ysa bwxr      " \
           "                                                                " \
           "                                                                "


class SeqFile(object):
    """
    A biological sequence is a sequence of bytes or characters.  Usually these
    represent DNA (A,C,G,T), proteins, or some variation of those.

    class attributes:

        file:    file object containing the sequence
        revcomp: whether gets from this sequence should be reverse-complemented
                 False => no reverse complement
                 True  => (same as "-5'")
                 "maf" => (same as "-5'")
                 "+5'" => minus strand is from plus strand's 5' end (same as "-3'")
                 "+3'" => minus strand is from plus strand's 3' end (same as "-5'")
                 "-5'" => minus strand is from its 5' end (as per MAF file format)
                 "-3'" => minus strand is from its 3' end (as per genome browser,
                          but with origin-zero)
        name:    usually a species and/or chromosome name (e.g. "mule.chr5");  if
                 the file contains a name, that overrides this one
        gap:     gap character that aligners should use for gaps in this sequence
    """

    def __init__(self, file=None, revcomp=False, name="", gap=None):

        self.file = file
        if revcomp:
            self.revcomp = "-5'"
        elif revcomp == "+3'":
            self.revcomp = "-5'"
        elif revcomp == "+5'":
            self.revcomp = "-3'"
        elif revcomp == "maf":
            self.revcomp = "-5'"
        else:
            self.revcomp = revcomp
        self.name = name
        if gap is None:
            self.gap = "-"
        else:
            self.gap = gap

        self.text = None  # (subclasses fill in text and
        self.length = 0  # length or they most override get())

    def close(self):
        assert (self.file is not None)
        self.file.close()
        self.file = None

    def extract_name(self, line):
        try:
            return line.split()[0]
        except Exception:
            return ""

    def set_text(self, text):
        self.text = text
        self.length = len(text)

    def __str__(self):
        text = ""
        if self.name is not None:
            text += self.name + " "
        text += self.get(0, self.length)
        return text

    def get(self, start, length):
        """
        Fetch subsequence starting at position `start` with length `length`.
        This method is picky about parameters, the requested interval must
        have non-negative length and fit entirely inside the NIB sequence,
        the returned string will contain exactly 'length' characters, or an
        AssertionError will be generated.
        """
        # Check parameters
        assert length >= 0, "Length must be non-negative (got %d)" % length
        assert start >= 0, "Start must be greater than 0 (got %d)" % start
        assert start + length <= self.length, \
            "Interval beyond end of sequence (%s..%s > %s)" % (start, start + length, self.length)
        # Fetch sequence and reverse complement if necesary
        if not self.revcomp:
            return self.raw_fetch(start, length)
        if self.revcomp == "-3'":
            return self.reverse_complement(self.raw_fetch(start, length))
        assert self.revcomp == "-5'", "unrecognized reverse complement scheme"
        start = self.length - (start+length)
        return self.reverse_complement(self.raw_fetch(start, length))

    def raw_fetch(self, start, length):
        return self.text[start:start+length]

    def reverse_complement(self, text):
        comp = [ch for ch in text.translate(DNA_COMP)]
        comp.reverse()
        return "".join(comp)


class SeqReader(Iterator):
    """Iterate over all sequences in a file in order"""

    def __init__(self, file, revcomp=False, name="", gap=None):
        self.file = file
        self.revcomp = revcomp
        self.name = name
        self.gap = gap
        self.seqs_read = 0

    def close(self):
        self.file.close()

    def __iter__(self):
        return SeqReaderIter(self)

    def __next__(self):  # subclasses should override this method and return the
        return   # .. next sequence (of type SeqFile or a subclass) read from self.file


class SeqReaderIter(Iterator):
    def __init__(self, reader):
        self.reader = reader

    def __iter__(self):
        return self

    def __next__(self):
        v = next(self.reader)
        if not v:
            raise StopIteration
        return v



"""
:Author: M. Simionato
:Date: April 2004
:Title: A much simplified interface to optparse.

You should use optionparse in your scripts as follows.
First, write a module level docstring containing something like this
(this is just an example)::

    '''usage: %prog files [options]
       -d, --delete: delete all files
       -e, --erase = ERASE: erase the given file'''

Then write a main program of this kind:

# sketch of a script to delete files::

    if __name__=='__main__':
        import optionparse
        option,args=optionparse.parse(__doc__)
        if not args and not option: optionparse.exit()
        elif option.delete: print "Delete all files"
        elif option.erase: print "Delete the given file"

Notice that ``optionparse`` parses the docstring by looking at the
characters ",", ":", "=", "\\n", so be careful in using them. If
the docstring is not correctly formatted you will get a SyntaxError
or worse, the script will not work as expected.
"""
from __future__ import print_function

import optparse
import re
import sys
import traceback

USAGE = re.compile(r'(?s)\s*usage: (.*?)(\n[ \t]*\n|$)')


def nonzero(self):  # will become the nonzero method of optparse.Values
    "True if options were given"
    for v in self.__dict__.values():
        if v is not None:
            return True
    return False


optparse.Values.__nonzero__ = nonzero  # dynamically fix optparse.Values


class ParsingError(Exception):
    pass


optionstring = ""


def exception(msg=""):
    print("Exception while parsing command line:", file=sys.stderr)
    print(traceback.format_exc(), file=sys.stderr)
    exit(msg)


def exit(msg=""):
    raise SystemExit(msg or optionstring.replace("%prog", sys.argv[0]))


def parse(docstring, arglist=None):
    global optionstring
    optionstring = docstring
    match = USAGE.search(optionstring)
    if not match:
        raise ParsingError("Cannot find the option string")
    optlines = match.group(1).splitlines()
    try:
        p = optparse.OptionParser(optlines[0], conflict_handler="resolve")
        for line in optlines[1:]:
            opt, help = line.split(':')[:2]
            # Make both short and long optional (but at least one)
            # Old: short,long=opt.split(',')[:2]
            opt_strings = []
            action = "store_true"
            for k in opt.split(', '):
                k = k.strip()
                if k.startswith("--") and "=" in k:
                    action = "store"
                    k = k.split("=")[0]
                opt_strings.append(k)
            p.add_option(*opt_strings, **dict(action=action, help=help.strip()))
    except (IndexError, ValueError):
        raise ParsingError("Cannot parse the option string correctly")
    return p.parse_args(arglist)


def help_callback(option, opt, value, parser, help):
    print(help, file=sys.stderr)
    sys.exit(1)



from cpython.version cimport PY_MAJOR_VERSION

cdef extern from "Python.h":
    char * PyBytes_AsString( object )
    object PyBytes_FromStringAndSize( char *, Py_ssize_t )

import struct, sys

cdef char * NIB_I2C_TABLE 
cdef char * NIB_I2C_TABLE_FIRST 
cdef char * NIB_I2C_TABLE_SECOND
#NIB_I2C_TABLE        = "TCAGNXXXtcagnxxx"
NIB_I2C_TABLE_FIRST  = "TTTTTTTTTTTTTTTTCCCCCCCCCCCCCCCCAAAAAAAAAAAAAAAAGGGGGGGGGGGGGGGGNNNNNNNNNNNNNNNNXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXttttttttttttttttccccccccccccccccaaaaaaaaaaaaaaaaggggggggggggggggnnnnnnnnnnnnnnnnxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
NIB_I2C_TABLE_SECOND = "TCAGNXXXtcagnxxxTCAGNXXXtcagnxxxTCAGNXXXtcagnxxxTCAGNXXXtcagnxxxTCAGNXXXtcagnxxxTCAGNXXXtcagnxxxTCAGNXXXtcagnxxxTCAGNXXXtcagnxxxTCAGNXXXtcagnxxxTCAGNXXXtcagnxxxTCAGNXXXtcagnxxxTCAGNXXXtcagnxxxTCAGNXXXtcagnxxxTCAGNXXXtcagnxxxTCAGNXXXtcagnxxxTCAGNXXXtcagnxxx"

def translate_raw_data( data, int start, int length ):
    """
    Data is a block read from the file that needs to be unpacked, dealing
    with end conditions based on start/length.
    """
    cdef int i, j
    cdef char * p_rval
    cdef unsigned char * p_data

    if length == 0 :
      return ""

    # Allocate string to write into
    rval = PyBytes_FromStringAndSize( NULL, length )

    # Get char pointer access to strings
    p_rval = PyBytes_AsString( rval )
    p_data = <unsigned char *> PyBytes_AsString( data )
    i = 0
    # Odd start
    if start & 1: 
        #p_rval[i] = NIB_I2C_TABLE[ p_data[0] & 0xF ]
        p_rval[i] = NIB_I2C_TABLE_SECOND[ p_data[0] ]
        p_data = p_data + 1
        i = 1
    # Two output values for each input value
    for j from 0 <= j < (length-i)/2:
        #p_rval[i]   = NIB_I2C_TABLE[ ( p_data[0] >> 4 ) & 0xF ];
        #p_rval[i+1] = NIB_I2C_TABLE[ ( p_data[0] >> 0 ) & 0xF ];
        p_rval[i]   = NIB_I2C_TABLE_FIRST [ p_data[0] ]
        p_rval[i+1] = NIB_I2C_TABLE_SECOND[ p_data[0] ]
        i = i + 2
        p_data = p_data + 1
    # Odd end
    if i < length: 
        p_rval[i] = NIB_I2C_TABLE_FIRST[ p_data[0] ]

    if PY_MAJOR_VERSION >= 3:
        return rval.decode()
    else:
        return rval



#!/usr/bin/env python

"""
Translate a maf file containing gap ambiguity characters as produced by
'maf_tile_2.py' to a new file in which "#" (contiguous) is replaced by "-" and
all other types are replaces by "*".

TODO: This could be much more general, should just take the translation table
      from the command line.

usage: %prog < maf > maf
"""

from __future__ import division

import string
import sys

from bx.align import maf

table = string.maketrans("#=X@", "-***")


def main():

    maf_reader = maf.Reader(sys.stdin)
    maf_writer = maf.Writer(sys.stdout)

    for m in maf_reader:
        for c in m.components:
            c.text = c.text.translate(table)
        maf_writer.write(m)

    maf_writer.close()


if __name__ == "__main__":
    main()



#!/usr/bin/env python
from __future__ import print_function

import logging
import sys
from collections import OrderedDict
from itertools import product

import numpy as np

from bx.align.epo import Chain, EPOitem
from bx.cookbook import argparse

logging.basicConfig(level=logging.INFO)
log = logging.getLogger()


def outFile(s):
    if (s in ('-', 'stdout')) or (s is None):
        return sys.stdout
    return open(s, 'w')


def loadChrSizes(path):
    data = OrderedDict()
    with open(path) as fd:
        for ch, s in (l.split() for l in fd):
            data[ch] = int(s)
    return data


def convert_action(trg_comp, qr_comp, ts, qs, opt):
    for i, (a, b) in enumerate(product(trg_comp, qr_comp)):
        try:
            ch, S, T, Q = Chain._make_from_epo(a, b, ts, qs)
            if np.sum(S) == 0:
                log.info("insignificant genomic alignment block %s ..." % ch.id)
                continue
            new_id = "%si%d" % (ch.id, i)
            print(str(ch._replace(id=new_id)), file=opt.output)
            for s, t, q in zip(S, T, Q):
                print("%d %d %d" % (s, t, q), file=opt.output)
            print("%d\n" % S[-1], file=opt.output)
        except KeyError:
            log.warning("skipping chromosome/contig (%s, %s)" % (a.chrom, b.chrom))


if __name__ == '__main__':
    parser = argparse.ArgumentParser(
        description="""EPO alignments (.out) to .chain converter.""",
        epilog="Olgert Denas (Taylor Lab)",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)

    parser.add_argument("input", help="File to process.")
    parser.add_argument("--species", nargs=2, default=["homo_sapiens", "mus_musculus"], help="Names of target and query species (respectively) in the alignment.")
    parser.add_argument("--chrsizes", nargs=2, required=True, help="Chromosome sizes for the given species.")
    parser.add_argument("-o", '--output', metavar="FILE", default='stdout', type=outFile, help="Output file")

    opt = parser.parse_args()

    log.info("loading sizes ...")
    tsizes = loadChrSizes(opt.chrsizes[0])
    qsizes = loadChrSizes(opt.chrsizes[1])

    log.info("loading alignments ...")
    data = OrderedDict(sorted(EPOitem._parse_epo(opt.input).items()))

    log.info("dumping ...")
    for k in data:
        components = data[k]
        trg_comp = [c for c in components if c.species == opt.species[0]]
        qr_comp = [c for c in components if c.species == opt.species[1]]

        convert_action(trg_comp, qr_comp, tsizes, qsizes, opt)



"""
Support for chopping a list of alignment blocks to only the portion that
intersects a particular interval.
"""


def chop_list(blocks, src, start, end):
    """
    For each alignment block in the sequence `blocks`, chop out the portion
    of the block that overlaps the interval [`start`,`end`) in the
    component/species named `src`.
    """
    new_blocks = []
    for block in blocks:
        ref = block.get_component_by_src(src)
        # If the reference component is on the '-' strand we should complement the interval
        if ref.strand == '-':
            slice_start = max(ref.src_size - end, ref.start)
            slice_end = max(ref.src_size - start, ref.end)
        else:
            slice_start = max(start, ref.start)
            slice_end = min(end, ref.end)
        sliced = block.slice_by_component(ref, slice_start, slice_end)
        good = True
        for c in sliced.components:
            if c.size < 1:
                good = False
        if good:
            new_blocks.append(sliced)
    return new_blocks



#!/usr/bin/env python

"""
Read a set of ranges and a nib file, print portions of nib overlapping
those ranges to stdout

usage: %prog range_file nib_file
"""
from __future__ import print_function

import bx.seq.nib
from bx.cookbook import doc_optparse


def __main__():

    options, args = doc_optparse.parse(__doc__)

    try:
        range_file = open(args[0])
        nib_file = open(args[1])
    except Exception:
        doc_optparse.exit()

    nib = bx.seq.nib.NibFile(nib_file)

    for line in range_file:
        fields = line.split()
        start, end = int(fields[0]), int(fields[1])
        print(">", start, end)
        print_wrapped(nib.get(start, end - start))


def print_wrapped(s):
    l = len(s)
    c = 0
    while c < l:
        b = min(c + 50, l)
        print(s[c:b])
        c = b


if __name__ == "__main__":
    __main__()
